// Angular
import { Injectable } from '@angular/core';

// NgRx
import { Store, select } from '@ngrx/store';
import { first } from 'rxjs/operators';
import { EventTransceiverState } from '../reducers/event-transceiver.reducer';
import {
  EventTransceiverReceiveClickEvent,
  EventTransceiverReceivePageLoadEvent,
  EventTransceiverReceiveSiteErrorEvent,
  EventTransceiverReceiveAddToCartEvent,
  EventTransceiverReceiveSuperSkuSwitchEvent,
  EventTransceiverSetWhitelistEvent,
  EventTransceiverReceiveCarouselItemClickEvent
} from '../actions';
import { WebPerformanceCalculateFirstMeaningfulPaint, WebPerformanceReturnStore } from '../actions/web-performance.actions';
import { selectEventTrackerEventLog } from '../selectors/event-transceiver.selectors';

// Interfaces and Constants
import * as constants from '../constants/adobe-analytics.constants';
import { EventTrackerLogEntry } from '../models/event-tracker.interface';

// Services
import { PayloadVerificationService } from './payload-verification.service';

/*
 * A subset of event tranceiver's functionalities are meant to be exposed to the outside  in order to allow non-angular entities to submit data to Adobe DTM using AdobeAnalyticsService.
 * This class EventTransceiverService is meant to be a wrapper of that subset of functionalities, and shall be attached to the Window object wherever it is needed
*/
@Injectable()
export class EventTransceiverService {

  constructor(
    private store: Store<EventTransceiverState>,
    private payloadVerificationService: PayloadVerificationService
  ) {
  }

  private handleIncorrectPayloadError(eventType) {
    console.log(`[Event Transceiver External] Incorrect payload received for event : ${eventType}`);
  }

  private handleIncorrectWebPerformancePayloadError() {
    console.log('[Event Transceiver External] Incorrect payload');
  }

  private handlePageLoadEvent(eventType, payload) {
    if (payload && this.payloadVerificationService.isPageAndModalViewPayload(payload)) {
      const event = {
        type: constants.eventTypes.pageLoad,
        page: constants.pageTypes.pip,
      };
      this.store.dispatch(new EventTransceiverReceivePageLoadEvent(event, payload.dataWrapper, payload.pageMeta));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private handleClickElementEvent(eventType, payload) {
    if (payload && this.payloadVerificationService.isElementClickPayload(payload)) {
      const event = {
        type: constants.eventTypes.pipClickElement,
        page: constants.pageTypes.pip,
      };
      this.store.dispatch(new EventTransceiverReceiveClickEvent(event, payload.elementType , null, payload.elementValue));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private handleSiteErrorEvent(eventType, payload) {
    if (payload && this.payloadVerificationService.isSiteErrorPayload(payload)) {
      this.store.dispatch(new EventTransceiverReceiveSiteErrorEvent(
        {
          type: constants.eventTypes.siteError
        },
        payload.errorType,
        payload.userInformed,
        payload.errorCode,
        payload.errorValue,
        payload.target,
        payload.errorPayload
    ));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private handleAddToCartEvent(eventType, payload) {
    if (payload && this.payloadVerificationService.isAddToCartPayload(payload)) {
      const event = {
        type: constants.eventTypes.pipAddToCart,
        page: constants.pageTypes.pip,
      };
      this.store.dispatch(new EventTransceiverReceiveAddToCartEvent(event, payload));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private handleSuperSkuSwitch(eventType, payload, page) {
    if (payload && this.payloadVerificationService.isSuperSkuSwitchPayload(payload)) {
      this.store.dispatch(new EventTransceiverReceiveSuperSkuSwitchEvent(
        page,
        payload.productId,
        payload.attributeId,
        payload.attributeName,
        payload.to_productId,
        payload.to_activeValueId,
        payload.to_activeValueName));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private handleCarouselItemClick(eventType, payload) {
    if (payload && this.payloadVerificationService.isCarouselItemClickPayload(payload)) {
      this.store.dispatch(new EventTransceiverReceiveCarouselItemClickEvent(payload));
    } else {
      this.handleIncorrectPayloadError(eventType);
    }
  }

  private removeWhitelistCallback() {
    if (window['EventTransceiver'] && window['EventTransceiver']['SetWhitelist']) {
      delete window['EventTransceiver']['SetWhitelist'];
    }
  }

  private createWhitelistCallback() {
    if (window['EventTransceiver']) {
      window['EventTransceiver']['SetWhitelist'] = (payload) => {
        if (payload && this.payloadVerificationService.isWhitelistPayload(payload)) {
          this.store.dispatch(new EventTransceiverSetWhitelistEvent(payload));
          this.removeWhitelistCallback();
        } else {
          this.handleIncorrectPayloadError('Set Whitelist');
        }
      };
    }
  }

  public attachToWindow () {
    if (typeof window !== 'undefined') {
      window['EventTransceiver'] = {
        'EventTrigger': this.eventTrigger.bind(this),
        'CalculateFirstMeaningfulPaint': this.updateFirstMeaningfulPaint.bind(this),
        'ReturnWebPerformanceStore': this.returnWebPerformanceStore.bind(this),
        'ReturnEventTrackerEventLog': this.returnEventTrackerEventLog.bind(this)
      };
    }
  }

  public eventTrigger (eventType: string, payload: any, page: string) {
    if (typeof window !== 'undefined' && eventType) {
      console.log(`[Event Tranceiver External] Event Received:  event = ${eventType}, payload = ${payload}`);
      switch (eventType) {
        case constants.eventTypes.pageLoad: this.handlePageLoadEvent(eventType, payload); break;
        case constants.eventTypes.pipClickElement: this.handleClickElementEvent(eventType, payload); break;
        case constants.eventTypes.siteError: this.handleSiteErrorEvent(eventType, payload); break;
        case constants.eventTypes.pipAddToCart: this.handleAddToCartEvent(eventType, payload); break;
        case constants.eventTypes.superSkuSwitch: this.handleSuperSkuSwitch(eventType, payload, page); break;
        case constants.eventTypes.carouselItemClick: this.handleCarouselItemClick(eventType, payload); break;
        default: console.log(`[Event Tranceiver External] Could not handle unrecognized external event: ${eventType}`);
      }
    }
  }

  public updateFirstMeaningfulPaint (componentPayload: Array<string>, httpPayloadFlag: boolean, httpPayload?: Array<string>) {
    if (typeof window !== 'undefined') {
      componentPayload = componentPayload || [];
      httpPayload = httpPayload || [];
      this.store.dispatch(new WebPerformanceCalculateFirstMeaningfulPaint(componentPayload, httpPayloadFlag, httpPayload));
    } else {
      this.handleIncorrectWebPerformancePayloadError();
    }
  }

  public returnWebPerformanceStore () {
    this.store.dispatch(new WebPerformanceReturnStore());
  }

  public initializeWhitelist() {
    if (window['EventTransceiverWhitelist']) {
      this.store.dispatch(new EventTransceiverSetWhitelistEvent(window['EventTransceiverWhitelist']));
    } else {
      this.createWhitelistCallback();
    }
  }

  public returnEventTrackerEventLog(): EventTrackerLogEntry[] {
    let log: EventTrackerLogEntry[];
    this.store.pipe(
      select(selectEventTrackerEventLog),
      first()
    )
    .subscribe((data: EventTrackerLogEntry[]) => {
      log = data;
    });
    return log;
  }
}
