import { Injectable, Inject, PLATFORM_ID } from '@angular/core';
import { Store, select } from '@ngrx/store';
const cloneDeep = require('lodash.clonedeep');
import { first } from 'rxjs/operators';
import { Subscription} from 'rxjs';
import { withLatestFrom } from 'rxjs/operators';
import { isPlatformBrowser } from '@angular/common';

// Event Transcevier stuff
import { WebPerformanceState } from '../reducers/web-performance.reducer';
import { Whitelist } from '../models/event-transceiver.interface';
import { ComponentRenderRecord, HttpReturnRecord } from '../models/web-performance.interface';
import { WEB_PERFORMANCE_TIME_FORMAT } from '../constants/web-performance-constants';
import { WebPerformanceUpdateRecord, WebPerformanceUpdateHttpRecord } from '../actions/web-performance.actions';
import { EventTransceiverState } from '../reducers/event-transceiver.reducer';
import {
  selectWebPerformance,
  selectHttpPerformance,
  selectComponentPerformance,
  selectWhitelist
} from '../selectors/event-transceiver.selectors';

// Others
import { DateTimeService } from '@hdca/fe-angular-shared';

// State Transfer
import { TransferState, makeStateKey } from '@angular/platform-browser';
import { EventTransceiverReceiveHTTPReturnEvent } from '../actions';
@Injectable()
export class WebPerformanceService {

  autoFMPCheckSub: Subscription;

  constructor(
    private dateTimeService: DateTimeService,
    private store: Store<EventTransceiverState>,
    private state: TransferState,
    @Inject(PLATFORM_ID) private platformId
  ) {
    isPlatformBrowser(this.platformId) && this.automaticFirstMeaningfulPaintTriggerDetect();
  }

  public processSSRHttpPerformanceList() {
    for (let i = 0; i >= 0; i++) {
      const item = this.state.get(makeStateKey(`HTTP_CALL${i}`), null);
      if (!!item) {
        this.store.dispatch(new EventTransceiverReceiveHTTPReturnEvent(item));
      } else {
        break;
      }
    }
  }

  public returnWebPerformanceState() {
    this.store.pipe(
      select(selectWebPerformance),
      first()
    ).subscribe(state => {
      console.log('web performance state is', state);
    });
  }

  public processRegisterComponentEvent(componentName: string, webPerformanceState: WebPerformanceState) {
    if (!!componentName && !!webPerformanceState) {
      const newRecordList = cloneDeep(webPerformanceState.ComponentRenderRecordList);
      const targetRecordExist = newRecordList.find((eachComponent) => {
        return eachComponent.name === componentName;
      });
      if (!targetRecordExist) {
        const newComponent: ComponentRenderRecord = {
          name: componentName,
          createTime: this.dateTimeService.getFormattedDate(new Date(), WEB_PERFORMANCE_TIME_FORMAT),
          completeTime: null,
          formattedCompleteTime: null,
          millisecondsFromStart: null
        };
        newRecordList.push(newComponent);
        this.store.dispatch(new WebPerformanceUpdateRecord(newRecordList, webPerformanceState.HttpReturnRecordList));
      }
    }
  }

  public processComponentRenderEvent(componentName: string, webPerformanceState: WebPerformanceState) {
    if (!!componentName && !!webPerformanceState) {
      const newRecordList = cloneDeep(webPerformanceState.ComponentRenderRecordList);
      const targetRecordExist = newRecordList.find((eachComponent) => {
        return eachComponent.name === componentName;
      });
      if (!!targetRecordExist) {
        const startTime = window && window['simulatedFirstPaint'] ? window['simulatedFirstPaint'] : new Date();
        targetRecordExist.millisecondsFromStart = this.dateTimeService.differenceInMilliseconds(new Date(), startTime);
        targetRecordExist.completeTime = Date.now();
        targetRecordExist.formattedCompleteTime = this.dateTimeService.getFormattedDate(new Date(), WEB_PERFORMANCE_TIME_FORMAT);
        this.store.dispatch(new WebPerformanceUpdateRecord(newRecordList, webPerformanceState.HttpReturnRecordList));
      }
    }
  }

  public processHTTPEvent(payload: HttpReturnRecord, httpReturnRecordList: HttpReturnRecord[]) {
      const instances = httpReturnRecordList.filter((record) => {
        return record.url === payload.url;
      });
      const startTime = window && window['simulatedFirstPaint'] ? window['simulatedFirstPaint'] : new Date();
      payload = {
        ...payload,
        instance : instances.length + 1,
        formattedCompleteTime: this.dateTimeService.getFormattedDate(payload.completeTime, WEB_PERFORMANCE_TIME_FORMAT)
      };
      this.store.dispatch(new WebPerformanceUpdateHttpRecord(payload));
  }

  public calculateFirstMeaningfulPaint(webPerformanceState: WebPerformanceState, componentPayload: Array<string>, httpPayloadFlag: boolean, httpPayload: Array<string>) {
    if (window && window['EventTransceiver']) {
      const startTime = window['simulatedFirstPaint'];
      if (componentPayload.length > 0 || httpPayloadFlag || httpPayload.length > 0) {
        const list: Array<any> = [];
        componentPayload.forEach((eachElement) => {
          const target = webPerformanceState.ComponentRenderRecordList.find((eachRecord) => {
            return eachRecord.name === eachElement;
          });
          !!target && list.push(target);
        });
        if (httpPayloadFlag) {
          webPerformanceState.HttpReturnRecordList.forEach((eachRecord) => {
            list.push(eachRecord);
          });
        } else {
          httpPayload.forEach((eachElement) => {
            const target = webPerformanceState.HttpReturnRecordList.forEach((eachRecord) => {
              if (!!eachRecord.url.includes(eachElement)) { // TODO: this logic needs to be better
                list.push(eachRecord);
              }
            });
          });
        }
        if (list.length > 0) { // last line of defense before data capture
          this.populateMetrics(list, startTime);
        }
      } else {
        window['EventTransceiver']['WebPerformance'] = null;
      }
    }
  }

  private populateMetrics(list, startTime) {
    const sortedList = list.sort((a, b) => {
      return b.completeTime - a.completeTime;
    });
    const lastCompleteItem = sortedList[0];
    const lastCompleteItemInMillis = lastCompleteItem.millisecondsFromStart || lastCompleteItem.responseTimeInMillis;
    const displayResult = `${parseInt(`${lastCompleteItemInMillis / 1000}`, 10)}.${lastCompleteItemInMillis % 1000} seconds`;
    window['EventTransceiver']['WebPerformanceMetrics'] = {
      'FirstContentfulPaint': startTime.getTime(),
      'FirstMeaningfulPaint': lastCompleteItem.completeTime,
      'FirstMeaningfulPaintDelta': lastCompleteItemInMillis
    };
    console.log('Event-Transceiver, calculating FMP, result = ', displayResult);
  }

  private automaticFirstMeaningfulPaintTriggerDetect() {
    this.autoFMPCheckSub = this.store.pipe(
      select(selectWebPerformance),
      withLatestFrom(
        this.store.pipe(select(selectWhitelist))
      )
    ).subscribe(([state, whitelist]: [WebPerformanceState, Whitelist]) => {
      if (!!state && !!whitelist) {
        const componentWhitelist = whitelist.pip.webPerformance.componentList;
        const httpWhitelist = whitelist.pip.webPerformance.httpList;
        if (!!this.automaticFirstMeaningfulPaintTriggerCheck(state, componentWhitelist, httpWhitelist)) {
          this.calculateFirstMeaningfulPaint(state, componentWhitelist, false, httpWhitelist);
          this.autoFMPCheckSub.unsubscribe();
        }
      }
    });
  }

  private automaticFirstMeaningfulPaintTriggerCheck(state: WebPerformanceState, componentWhitelist: Array<any>, httpWhitelist: Array<any>): boolean {
    if (window['EventTransceiver'] &&
      (!window['EventTransceiver']['WebPerformanceMetrics'] || !window['EventTransceiver']['WebPerformanceMetrics']['FirstMeaningfulPaint'])
      ) {
      if (componentWhitelist.length > 0 || httpWhitelist.length > 0) {
        for (let componentKey = 0; componentKey < componentWhitelist.length; ++componentKey) {
          const targetComponentName = componentWhitelist[componentKey];
          const foundTarget = state.ComponentRenderRecordList.find((eachRecord) => {
              return eachRecord.name === targetComponentName;
          });
          if (!foundTarget || !foundTarget['completeTime']) {
            return false;
          }
        }
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }
}
