import { IRecommendations, IRecommendationItem } from './../../models/recommendations.interface';
import { Component, OnInit, OnDestroy } from '@angular/core';
import { State } from '../../../../reducers';
import { select, Store } from '@ngrx/store';
import { LanguageService } from '@hdca/fe-angular-shared';
import { Observable, Subscription } from 'rxjs';
import { GlobalCartService } from '../../services/global-cart.service';
import { GlobalCartDataState } from '../../reducers/global-cart-data/global-cart-data.reducer';
import { GlobalCartAddItemDataState } from '../../reducers/global-cart-add-item-data/global-cart-add-item-data.reducer';
import { distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GLOBAL_UI_CONSTANTS } from '@hdca/fe-angular-shared';
import { HideAddToCartSuccess } from '../../actions/global-cart-ui/global-cart-ui.actions';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { GetConfiguration } from '../../actions/global-cart-paypal-data/global-cart-paypal-data.actions';
import { selectGlobalCartPaypalDataState } from '../../selectors/global-cart-paypal-data.selectors';
import { GlobalCartPaypalDataState } from '../../reducers/global-cart-paypal-data/global-cart-paypal-data.reducer';
import { ApiParamInfoService, TransformUrlPipe } from '@hdca/fe-angular-shared';
import * as recommendationActions from '../../actions/recommendations/recommendation.actions';
import { scrollToTop } from '../../../../utils/scrollUtil';
import { productInteractionLocationTypes, pageTypes } from '../../../event-transceiver/constants/adobe-analytics.constants';
import { EventTransceiverReceiveCarouselItemClickEvent } from '../../../event-transceiver/actions/event-transceiver.actions';
import { ProductClickBlock } from '../../../event-transceiver/models/adobe-analytics-data-layer-models.interface';

@Component({
  selector: 'add-to-cart-success-container',
  templateUrl: 'global-add-to-cart-success.container.html'
})
export class GlobalAddToCartSuccessContainer implements OnInit, OnDestroy {

  // Fields
  language: string;
  GLOBAL_ICONS_SVG_PATH = GLOBAL_UI_CONSTANTS.GLOBAL_ICONS_SVG_PATH;

  // Observables
  cartTranslation$: Observable<string>;
  checkoutTranslation$: Observable<string>;
  currentCartData$: Observable<GlobalCartDataState>;
  cartAddedItemData$: Observable<GlobalCartAddItemDataState>;
  switchedCartData$: Observable<{ addedItemData: GlobalCartAddItemDataState, currentCartData: GlobalCartDataState }>;
  paypalCheckout$: Observable<GlobalCartPaypalDataState>;
  cartRecommendations$: Observable<IRecommendationItem[]>;

  addedItemSub: Subscription;
  contextProductId: string;
  productIdSub: Subscription;

  constructor(private store: Store<State>,
    private languageService: LanguageService,
    private globalCartService: GlobalCartService,
    private router: Router,
    private translateService: TranslateService,
    private apiParams: ApiParamInfoService,
    private transformUrl: TransformUrlPipe
  ) {
  }

  ngOnInit() {
    this.language = this.languageService.getLanguage();
    this.initObservables();
    this._getAlsoPurchasedItems();
  }

  ngOnDestroy() {
    this.store.dispatch(new HideAddToCartSuccess());
    this.addedItemSub.unsubscribe();
    this.productIdSub.unsubscribe();
  }

  /**
   * Initialization - The methods below are related to initilization of component variables
   */

  private initObservables() {
    // Data stream to get the translated cart url
    this.cartTranslation$ = this.translateService.get('ATC_SUCCESS.CART');

    // Data stream to get the translated checkout url
    this.checkoutTranslation$ = this.translateService.get('ATC_SUCCESS.CHECKOUT');

    // Data stream to get the recently added to cart item
    this.cartAddedItemData$ = this.globalCartService.getGlobalCartAddItemState$();

    // Data stream to get the current cart
    this.currentCartData$ = this.globalCartService.getGlobalCartDataState$();

    this.cartRecommendations$ = this.globalCartService.getRecommendations$();

    // Sub to get the product Id which is required in all dispatches
    this.productIdSub = this.globalCartService.getCurrentProductId$().subscribe(productId => {
      this.contextProductId = productId;
    });

    // Data stream to switchMap from added item to current cart
    this.switchedCartData$ = this.cartAddedItemData$.pipe(
      switchMap(() => {
        return this.currentCartData$;
      }, (addedItemData, currentCartData) => {
        return { addedItemData, currentCartData };
      }),
      distinctUntilChanged()
    );

    // Data stream for Checkout with Paypal
    this.paypalCheckout$ = this.store.pipe(
      select(selectGlobalCartPaypalDataState),
      distinctUntilChanged()
    );

    this.addedItemSub = this.cartAddedItemData$.subscribe((data) => {
      if (data.entry && data.entry.product) {
        this.contextProductId = data.entry.product.code;
      }
    });
  }

  // get the customers also purchased in the sucess popup
  private _getAlsoPurchasedItems() {
    const isCartSchema = true;
    const { localizedPOS, trackingId, catalogVersion, lang, scheme, relationship, full } = this.apiParams.getApiParamInfo(isCartSchema);
    const productId = this.contextProductId;
    const payload: IRecommendations = {
      productId,
      localizedPOS,
      trackingId,
      catalogVersion,
      lang,
      scheme,
      relationship,
      fields: full
    };
    this.store.dispatch(new recommendationActions.GetRecommendations(payload));
  }

  /**
   * Events - The methods below are related to responding to events from template
   */

  onCloseSuccessModal() {
    this.store.dispatch(new HideAddToCartSuccess());
  }

  onClickProceedToCheckout(checkoutTranslation) {
    window.location.href = checkoutTranslation;
  }

  onClickPaypal() {
    this.store.dispatch(new GetConfiguration(this.language));
  }

  onClickViewCart(cartTranslation) {
    this.router.navigateByUrl(cartTranslation);
  }

  onClickContinueShopping() {
    this.store.dispatch(new HideAddToCartSuccess());
  }

  addToCart($event, index) {
    if ($event && $event.code) {
      this.globalCartService
        .addItemToCart$($event.optimizedPrice.displayPrice.value, $event.code, false, index + 1, productInteractionLocationTypes.modalCertona)
        .subscribe()
        .unsubscribe();
    }
  }

  toggleProduct($event, index) {
    this.analyticsTrackCertonaItemClick($event, index);
    const reviews = $event.reviews;
    const productTranslation = this.translateService.instant('PRODUCT');
    const suffix = $event.goToReviews ? `?reviews=true` : '';
    const url: string = $event && $event.url ? $event.url : null;
    if (url.includes('.html')) {
        const hidePrefix = true;
        const goToUrl = `${productTranslation}/${this.transformUrl.transform(url, hidePrefix)}${suffix}`;
        this.router.navigateByUrl(goToUrl);
    } else {
      this.router.navigateByUrl(url);
    }
    if (reviews) {
      // todo make this a global action
    } else {
      scrollToTop();
    }
    this.store.dispatch(new HideAddToCartSuccess());
  }

  public analyticsTrackCertonaItemClick(event: any, index: number): void {
    if (event.code && index >= 0 && this.contextProductId) {
      const payload: ProductClickBlock = {
        productId: event.code,
        contextProductId: this.contextProductId,
        slotLocation: index + 1,
        productInteractionLocation: productInteractionLocationTypes.modalCertona,
        productInteractionPageType: pageTypes.pip,
        productInteractionLocationTitle: this.translateService.instant('ATC_SUCCESS.ADDED_TO_CART')
      };
      this.store.dispatch(new EventTransceiverReceiveCarouselItemClickEvent(payload));
    }
  }
}
