import { Component, OnDestroy, OnInit } from '@angular/core';
import { Observable, Subscription } from 'rxjs';
import { GlobalCartService } from '../../services/global-cart.service';
import { State } from '../../../../reducers';
import { select, Store } from '@ngrx/store';
import { GlobalCartUIState } from '../../reducers/global-cart-ui/global-cart-ui.reducer';
import { selectGlobalCartUIState } from '../../selectors/global-cart-ui.selectors';
import { GlobalCartSessionDataState } from '../../reducers/global-cart-session-data/global-cart-session-data.reducer';
import { selectGlobalCartSessionDataState } from '../../selectors/global-cart-session-data.selectors';
import { CookiesService, LanguageService } from '@hdca/fe-angular-shared';
import { tap } from 'rxjs/operators';
import {
  CreateAccountCartSession,
  CreateAnonymousCartSession,
  SaveCartSessionByCookie
} from '../../actions/global-cart-session-data/global-cart-session-data.actions';
import { HttpHeaders } from '@angular/common/http';
import { GetCurrentCartData } from '../../actions/global-cart-data/global-cart-data.actions';

@Component({
  selector: 'global-cart-container',
  templateUrl: 'global-cart.container.html'
})
/**
 * @description - This is the root container for global cart. Include this container anywhere you wish to get cart data.
 */
export class GlobalCartContainer implements OnInit, OnDestroy {

  // Fields
  language: string;
  headers: HttpHeaders;

  // Observables
  cartUI$: Observable<GlobalCartUIState>;
  cartSessionData$: Observable<GlobalCartSessionDataState>;

  // Subscriptions
  cartSessionDataSub: Subscription;

  constructor(private globalCartService: GlobalCartService, private store: Store<State>, private languageService: LanguageService, private cookiesService: CookiesService) {
  }

  ngOnInit() {
    this.language = this.languageService.getLanguage();
    this.initObservables();
    this.initSubscriptions();
  }

  ngOnDestroy() {
    this.cartSessionDataSub.unsubscribe();
  }

  /**
   * Initialization - The methods below are related to initilization of component variables
   */

  private initObservables() {

    // Stream to get the UI state.
    // Used to show or hide certain modals.
    this.cartUI$ = this.store.pipe(
      select(selectGlobalCartUIState)
    );

    // Stream to get the current cart session.
    // Also populates the store with cart id from cookie if none exists, or with new anonymous cart session if no cookie
    this.cartSessionData$ = this.store.pipe(
      select(selectGlobalCartSessionDataState),
      tap((session) => {
        if (!session.cart) {
          const cartCookie = this.cookiesService.getCookie('cart');
          const emailCookie = this.cookiesService.getCookie('email');
          if (!!cartCookie) {
            this.store.dispatch(new SaveCartSessionByCookie(cartCookie));
          } else {
            if (!emailCookie) {
              this.store.dispatch(new CreateAnonymousCartSession(this.language));
            } else {
              this.store.dispatch(new CreateAccountCartSession(this.language, emailCookie));
            }
          }
        } else {
          this.store.dispatch(new GetCurrentCartData(this.languageService.getLanguage()));
        }
      }),
    );
  }

  private initSubscriptions() {
    this.cartSessionDataSub = this.cartSessionData$.subscribe();
  }
}
