import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, mergeMap, switchMap, tap, filter } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import {  DataSender } from '../../../../services/data-sender/data-sender.service';
import { OriginService } from '../../../../services/origin/origin.service';
import { GetCurrentCartData } from '../../actions/global-cart-data/global-cart-data.actions';
import { LanguageService } from '@hdca/fe-angular-shared';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {
  AddApplianceItemToCart, ErrorAddedApplianceItemData,
  GlobalCartAddApplianceItemDataActionTypes, SaveAddedApplianceItemData
} from '../../actions/global-cart-add-appliance-item-data/global-cart-add-appliance-item-data.actions';
import { EventTransceiverReceiveAddToCartEvent } from '../../../event-transceiver/actions/event-transceiver.actions';
import { eventTypes, pageTypes, shippingMethodTypes, productInteractionLocationTypes } from '../../../event-transceiver/constants/adobe-analytics.constants';
import { AddToCartInfoBlock } from '../../../event-transceiver/models/adobe-analytics-data-layer-models.interface';
import { GlobalCartPartsAndServicesDataInterface } from '../../models/global-cart-parts-and-services-data.interface';

@Injectable()
export class GlobalCartAddApplianceItemDataEffects {
  productCode: string;
  productName: string;
  @Effect()
  postAddApplianceToCart$ = this.actions$
    .pipe(
      ofType<AddApplianceItemToCart>(GlobalCartAddApplianceItemDataActionTypes.AddApplianceItemToCart),
      switchMap((action) => {
        return this.dataSender.post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.POST_ADD_APPLIANCE_TO_CART(action.user, action.cartId, action.postalCode, action.localizedPOS), action.lang, action.partsAndServices).pipe(
          mergeMap(addedItem => {
            const { EventBlock, analyticsData } = this.generateEventTransceiverPayload(action.partsAndServices, addedItem);
            return [
              new EventTransceiverReceiveAddToCartEvent(EventBlock, analyticsData),
              new SaveAddedApplianceItemData(addedItem, action.addOnAvailable),
              /**
               * EACH TIME YOU ADD ITEM TO CART, YOU NEED TO UPDATE THE CART SO THAT IT UPDATES IN ANY PLACE THAT IS SUBSCRIBED TO CART REDUCER.
               * THIS IS IMPORTANT, AS ANY NEW ADD TO CART ACTIONS THAT MISS THIS STEP, WILL CAUSE STALE CART DATA
               */
              new GetCurrentCartData(this.langService.getLanguage()),
            ];
          }),
          catchError(error => {
            return of(new ErrorAddedApplianceItemData());
          })
        );
      })
    );

  @Effect({dispatch: false})
  navigateToCart$ = this.actions$
    .pipe(
      ofType<SaveAddedApplianceItemData>(GlobalCartAddApplianceItemDataActionTypes.SaveAddedApplianceItemData),
      switchMap((action) => {
        return this.generateRedirectUrl(action.payload, action.addOnAvailable);
      }),
      filter(cartTranslation => !!cartTranslation),
      tap(cartTranslation => {
        if (cartTranslation.includes('productCode')) {
          cartTranslation = cartTranslation.replace('{productCode}', this.productCode).replace('{productName}', this.productName);
          if (typeof window !== 'undefined') {
            window.location.href = this.originService.getOrigin() + cartTranslation;
          }
        } else {
          this.router.navigateByUrl(cartTranslation);
        }
      })
    );

  constructor(private actions$: Actions, private dataSender: DataSender, private originService: OriginService, private langService: LanguageService, private router: Router, private translateService: TranslateService) {
  }

  generateRedirectUrl(addedItem, addOnAvailable: boolean) {
    if (!!addOnAvailable && !!addedItem && addedItem.applianceModification) {
      this.productCode = addedItem.applianceModification.entry.product.code;
      this.productName = addedItem.applianceModification.entry.product.name;
      return this.translateService.get('ATC_SUCCESS.PRODUCT_ADDON_AEM_URL');
    } else {
      return this.translateService.get('ATC_SUCCESS.CART');
    }
  }

  private generateEventTransceiverPayload(partsAndServices: GlobalCartPartsAndServicesDataInterface, addedItem) {
    if (!!partsAndServices) {
      let analyticsData: AddToCartInfoBlock[];
      let ISAnalyticsData: AddToCartInfoBlock;
      const EventBlock = {
        type: eventTypes.pipAddToCart,
        page: pageTypes.pip
      };
      analyticsData = [{
        product: { code: partsAndServices.appliance.code },
        displayPrice: partsAndServices.appliance.price.formattedValue,
        itemQuantity: 1,
        isCartSwitch: false,
        displayShippingMethod: shippingMethodTypes.appliance,
        productInteractionLocation: productInteractionLocationTypes.primary,
        slotLocation: 1
      }];

      if (!!addedItem && addedItem.applianceAddonModification && addedItem.applianceAddonModification.length > 0) {
        ISAnalyticsData = {
          product: { code: addedItem.applianceAddonModification[0].entry.product.code },
          displayPrice: addedItem.applianceAddonModification[0].entry.displayPrice.formattedValue,
          itemQuantity: addedItem.applianceAddonModification[0].quantity,
          isCartSwitch: false,
          displayShippingMethod: shippingMethodTypes.appliance,
          productInteractionLocation: productInteractionLocationTypes.primary,
          slotLocation: 2
        };
        analyticsData.push(ISAnalyticsData);
      }

      return { EventBlock, analyticsData };
    }
  }
}
