import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, map, mergeMap, switchMap } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import { OriginService } from '../../../../services/origin/origin.service';
import { DataSender } from '../../../../services/data-sender/data-sender.service';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { HttpHeaders } from '@angular/common/http';
import {
  GlobalCartAddFbtDataActionTypes,
  AddItemsToCart,
  SaveAddedItemsData,
  AddItemsToCartError
} from '../../actions/global-cart-add-item-data/global-cart-add-fbt-data.actions';
import { GetCurrentCartData } from '../../actions/global-cart-data/global-cart-data.actions';
import { LanguageService } from '@hdca/fe-angular-shared';
import { GlobalCartService } from '../../services/global-cart.service';

// Event Transceiver
import { AddToCartInfoBlock } from '../../../event-transceiver/models/adobe-analytics-data-layer-models.interface';
import { productInteractionLocationTypes, eventTypes, pageTypes, shippingMethodTypes } from '../../../event-transceiver/constants/adobe-analytics.constants';
import { EventTransceiverReceiveAddToCartEvent } from '../../../event-transceiver/actions/event-transceiver.actions';

@Injectable()
export class GlobalCartAddFbtDataEffects {

  @Effect()
  postFbtAddToCart$ = this.actions$
    .pipe(
      ofType<AddItemsToCart>(GlobalCartAddFbtDataActionTypes.AddItemsToCart),
      switchMap((action) => {
        // Data for FBT is sent as a string representing JSON data
        const headers = new HttpHeaders({
          'Content-Type': 'application/json'
        });

        return this.dataSender.post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.POST_ADD_TO_CART(action.user, action.cartId, action.localizedPOS), action.lang, action.products, headers).pipe(
          mergeMap(cartData => {
            const jsonPayload = JSON.parse(action.products);
            const analyticsData: AddToCartInfoBlock[]  = jsonPayload.entries.
              map((product) => {
                return  {
                  product: { code: product.code},
                  itemQuantity: product.qty,
                  displayPrice: product.price,
                  displayShippingMethod: shippingMethodTypes.shipToHome, // FBT don't have fulfillmentMessages to capture shipping type, so analytics wants to pass default shippingMethod: 'STH'
                  isCartSwitch: false,
                  productInteractionLocation: productInteractionLocationTypes.fbt,
                  slotLocation: product.index,
                };
              });
            const EventBlock = {
              type: eventTypes.pipAddToCart,
              page: pageTypes.pip
            };
            return [
              cartData.cartModifications ? new SaveAddedItemsData(cartData) : new SaveAddedItemsData({cartModifications: []}),
              /**
               * EACH TIME YOU ADD ITEM TO CART, YOU NEED TO UPDATE THE CART SO THAT IT UPDATES IN ANY PLACE THAT IS SUBSCRIBED TO CART REDUCER.
               * THIS IS IMPORTANT, AS ANY NEW ADD TO CART ACTIONS THAT MISS THIS STEP, WILL CAUSE STALE CART DATA
               */
              new GetCurrentCartData(this.langService.getLanguage()),
              new EventTransceiverReceiveAddToCartEvent(EventBlock, analyticsData)
            ];
          }),
          catchError(error => {
            return of(new AddItemsToCartError(error, true));
          })
        );
      })
    );

  constructor(private actions$: Actions, private dataFetcher: DataFetcher, private dataSender: DataSender, private originService: OriginService, private langService: LanguageService, private globalCartService: GlobalCartService) {
  }

}
