import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, map, mergeMap, switchMap, tap } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import { OriginService } from '../../../../services/origin/origin.service';
import { DataSender } from '../../../../services/data-sender/data-sender.service';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { HideAddToCartSuccess, ShowAddToCartSuccess } from '../../actions/global-cart-ui/global-cart-ui.actions';
import { HttpHeaders } from '@angular/common/http';
import {
  GlobalCartAddItemDataActionTypes,
  AddItemToCart,
  SaveAddedItemData,
  AddCartError
} from '../../actions/global-cart-add-item-data/global-cart-add-item-data.actions';
import { GetCurrentCartData } from '../../actions/global-cart-data/global-cart-data.actions';
import { LanguageService } from '@hdca/fe-angular-shared';
import { Store } from '@ngrx/store';
import { State } from '../../../../reducers';

// Event Transceiver
import { eventTypes, pageTypes } from '../../../event-transceiver/constants/adobe-analytics.constants';
import { EventTransceiverReceiveAddToCartEvent } from '../../../event-transceiver/actions/event-transceiver.actions';
import { AclNotificationService, ACL_NOTIFICATIONS_CONTAINERS, AclAlertType } from '@hdca/fe-angular-cl';

@Injectable()
export class GlobalCartAddItemDataEffects {

  @Effect()
  postAddToCart$ = this.actions$
    .pipe(
      ofType<AddItemToCart>(GlobalCartAddItemDataActionTypes.AddItemToCart),
      switchMap((action) => {
        const headers = new HttpHeaders({
          'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
        });

        return this.dataSender.post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.POST_ADD_TO_CART(action.user, action.cartId, action.localizedPOS), action.lang, action.params, headers).pipe(
          tap(cartData => {
            if (cartData.quantityAdded === 0) {
              this.store.dispatch(new HideAddToCartSuccess());
            }
          }),
          mergeMap(cartData => {
            const EventBlock = {
              type: eventTypes.pipAddToCart,
              page: pageTypes.pip
            };
            this.checkQtyReduced(cartData);

            return [
              new SaveAddedItemData(cartData),
              /**
               * EACH TIME YOU ADD ITEM TO CART, YOU NEED TO UPDATE THE CART SO THAT IT UPDATES IN ANY PLACE THAT IS SUBSCRIBED TO CART REDUCER.
               * THIS IS IMPORTANT, AS ANY NEW ADD TO CART ACTIONS THAT MISS THIS STEP, WILL CAUSE STALE CART DATA
               */
              new GetCurrentCartData(this.langService.getLanguage()),
              new EventTransceiverReceiveAddToCartEvent(EventBlock, action.analyticsData)
            ];

          }),
          catchError(error => {
            // Hide the add to cart success modal if process has error
            this.store.dispatch(new HideAddToCartSuccess());
            // Required observable return for catchError method
            return of(new AddCartError(error, true));
          })
        );
      })
    );

  // IF MORE ACTIONS NEED TO HAVE SHOW SUCCESS MODAL EFFECT, ADD AS ADDITIONAL TYPE AS FOLLOWS:
  // ofType(ACTION1, ACTION2, ACTION3, etc)
  // IF NEW ACTION NEEDS TO HAVE A DIFFERENT EFFECT (such as a different modal), ADD AS A NEW EFFECT
  @Effect()
  showSuccessModal$ = this.actions$
    .pipe(
      ofType(GlobalCartAddItemDataActionTypes.AddItemToCart),
      map(() => new ShowAddToCartSuccess())
    );

  constructor(private actions$: Actions,
    private dataFetcher: DataFetcher,
    private dataSender: DataSender,
    private originService: OriginService,
    private langService: LanguageService,
    private store: Store<State>,
    private notificationService: AclNotificationService) {
  }

  private checkQtyReduced(cartData: any): void {
      if (cartData.quantity && cartData.message) {
          if (cartData.quantityAdded < cartData.quantity) {
              this.notificationService.addAlert({
                id: 'CART_QTY_REDUCE_INFO',
                type: AclAlertType.INFO,
                message: cartData.message,
                hasIcon: true,
                dismissible: true,
                animate: true,
                dismissTimeOut: 5000,
                containerId: ACL_NOTIFICATIONS_CONTAINERS.GLOBAL
            });
          }
      }
  }

}
