// Core imports
import { Component, OnInit, OnDestroy, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
// App Imports
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
import { LanguageService } from '@hdca/fe-angular-shared';
import { UserInfoService } from '@hdca/fe-angular-shared';
import { AuthenticationResponseConstants } from '@hdca/fe-angular-shared';
// Model imports
import { AuthenticationState, MyAccountNavNode } from '../../../models/global-header-account.interface';
// Ngrx imports
import { Subscription } from 'rxjs/Subscription';
import { Store } from '@ngrx/store';
import * as fromApp from '../../.../../../../../reducers';
import * as fromGlobalHeaderAccount from '../../../reducers/global-header-account/global-header-account.reducer';
import * as globalHeaderAccountActions from '../../../actions/global-header/global-header-account/global-header-account.action';
// Ngx imports
import { TranslateService } from '@ngx-translate/core';
// Rxjs imports
import { Observable } from 'rxjs/Observable';

@Component({
  selector: 'global-header-account',
  templateUrl: './global-header-account.container.html'
})
export class GlobalHeaderAccountContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
    // Set base values for UI state
    emailAddress: string;
    password: string;
    rememberMe: boolean;
    emailError: boolean;
    passwordError: boolean;
    hasValidationCompleted: boolean;
    loggingIn: boolean;
    revealPassword: boolean;
    // UI flags determined by cookies, but not needed for state since they are single use (never updated within the front end application)
    firstName: string;
    isAuth: boolean;
    isExpired: boolean;
    proxtraUserLoggedin: boolean;
    associateUserLoggedin: boolean;
    lastLoginOn: string;

    // Data from translation files
    createAccountItems: Array<string>;

    // Reset password redirect URL
    resetPasswordRedirect: string;

    // Observable for authentication alert
    // - If authentication alert occurs, just need to render a new Authentication Error section in the sign in form
    authenticationMessage$: Observable<AuthenticationState>;

    // Get my account nav nodes
    @Input() nodes: MyAccountNavNode;

    // Inform parent header that My Account close has been clicked
    @Output() closeMyAccount: EventEmitter<any> = new EventEmitter();

    constructor(
        private store: Store<fromApp.State>,
        private langService: LanguageService,
        private ref: ChangeDetectorRef,
        private translate: TranslateService,
        private user: UserInfoService) {
        super();
    }

    ngOnInit() {
        // Get data from translation service for account features
        this.initAccountList();
        // Get data from translation service for reset password URL
        this.getResetPasswordRedirect();
        // Get user data
        const user = this.user.getUserAccountData();
        const savedEmailAddress = user.savedEmailAddress;
        // Set the email address and remember me components of state before subscription
        this.store.dispatch(new globalHeaderAccountActions.RememberMeCheck(savedEmailAddress));
        // Subscribe to store data concerning global header account flyout
        const globalHeaderAccountSub: Subscription = this.store.select(fromApp.getGlobalHeaderAccount).
            subscribe((state: fromGlobalHeaderAccount.GlobalHeaderAccountState) => {
                // Subscription data to component data goes here
                this.emailAddress = state.emailAddress;
                this.password = state.password;
                this.rememberMe = state.rememberMe;
                this.emailError = state.emailError;
                this.passwordError = state.passwordError;
                this.hasValidationCompleted = state.hasValidationCompleted;
                this.loggingIn = state.loggingIn;
                this.revealPassword = state.revealPassword;
                // Handle change updating
                if (!this.ref['destroyed']) {
                    this.ref.detectChanges();
                }
            });
        // Subscribe to the global header account store changes
        this.subscriptions.push(globalHeaderAccountSub);
        //  Set one time use UI properties
        this.firstName = user.givenName ? user.givenName[0].toUpperCase() + user.givenName.slice(1) : '';
        this.isAuth = user.isAuth;
        this.isExpired = user.isExpired;
        this.proxtraUserLoggedin = user.proxtraUserLoggedin;
        this.associateUserLoggedin = user.associateUserLoggedin;
        this.lastLoginOn = user.lastLoginOn;
        // TODO: replace with actual data input from Global Header parent once available
        if (this.isAuth && !this.isExpired) {
            // User is signed in
            const accountLinks = this.getUserNavNode(this.proxtraUserLoggedin, this.associateUserLoggedin, this.nodes);
            this.nodes = accountLinks;
        } else {
            // User is not signed in, anonymous user
            const accountLinks = this.getAnonymousUserNode(this.nodes);
            this.nodes = accountLinks;
        }
        // Have the authentication section update depending on any errors returned
        this.authenticationMessage$ = this.store.map((state) => {
            // Get authentication info object
            const authenticationInfo = state.globalHeaderAccount.authenticationInfo;
            if (authenticationInfo) {
                // Get the authentication code
                const authenticationCode = authenticationInfo.code;
                // Depending on the authenication status, either perform reload, redirect or update state
                switch (authenticationCode) {

                    case AuthenticationResponseConstants.ACCEPTED:
                        // Reload page if login is accepted
                        if (typeof window !== 'undefined') {
                            window.location.reload(true);
                        }
                        return state.globalHeaderAccount.authenticationInfo;

                    case AuthenticationResponseConstants.RESET_PASSWORD:
                        // Redirect user to password reset page;
                        if (typeof window !== 'undefined') {
                            window.location.replace(this.resetPasswordRedirect);
                        }
                        return state.globalHeaderAccount.authenticationInfo;

                    default:
                        // Simply return existing authentication info if not performing success or password reset page change
                        return state.globalHeaderAccount.authenticationInfo;

                }
            } else {
                // If there is no authentication data yet, simply return the authenticationInfo state parameter
                return state.globalHeaderAccount.authenticationInfo;
            }

        });
        // Handle session expiry before anything else happens
        this.handleUserSessionExpiry();
    }

    handleUserSessionExpiry() {
        // Handle situation where My Account flyout loads and the user is expired
        if (this.isExpired) {
            // Set authentication alert, expired
            this.store.dispatch(new globalHeaderAccountActions.SetExpiredUser());
            // Clear expiration in cookie
            this.user.clearUserExpiry();
        }
    }

    initAccountList() {
        // Get the account benefits list from translation service and add it to the component data for display
        const initSub: Subscription = this.translate.get('GLOBAL_HEADER.MY_ACCOUNT.CREATE_ACCOUNT_LIST').subscribe(list => {
            this.createAccountItems = list;
        });

        this.subscriptions.push(initSub);
    }

    getResetPasswordRedirect() {
        // Get the reset password redirect url from the translation file
        const translateSub: Subscription = this.translate.get('GLOBAL_HEADER.MY_ACCOUNT.RESET_PASSWORD_REDIRECT').subscribe(url => {
            this.resetPasswordRedirect = url;
        });
        this.subscriptions.push(translateSub);
    }

    getAnonymousUserNode(data) {
        // Gets the links for an anonymous user (display on left hand side of My Account flyout)
        return data && data.children && data.children.filter(each => each.uid === 'MyAccountAnonymousNavNode')[0];
    }

    getUserNavNode (proxtraUserLoggedin, associateUserLoggedin, data) {
        // Gets the nav node for the user, depending on the user type
        // These are the links after sign in
        // TODO: replace with data from Nav Node store once it is merged
        if (associateUserLoggedin) {
            return data.children.filter(each => each.uid === 'MyAccountAssociateNavNode')[0];
        } else if (proxtraUserLoggedin) {
            return data.children.filter(each => each.uid === 'MyAccountProXtraNavNode')[0];
        } else {
            return data.children.filter(each => each.uid === 'MyAccountSignedInNavNode')[0];
        }

    }

    handleEmailAddressChange(event) {
        // Method called when the email address in the sign in section changes
        // Dependency: validateEmailAddress()
        const emailAddress = event.value;
        // Check if validation has been completed
        const previousState = this.hasValidationCompleted;
        // Only update the email here if the input has previously been validated (login attempted but validation failed)
        const error = previousState ? !this.validateEmailAddress(emailAddress) : false;
        // Dispatch email updates to the store
        this.store.dispatch(new globalHeaderAccountActions.ChangedEmailAddress(emailAddress, error));
    }

    validateEmailAddress(emailAddress) {
        // Helper method for checking that the latest entry for email address is valid
        const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;
        return emailAddress && emailAddress.match(emailRegex);
    }

    handlePasswordChange(event) {
        // Method called when the password in the sign in section changes
        // Dependency: validatePassword()
        const password = event.value;
        // Check if validation has been completed
        const previousState = this.hasValidationCompleted;
        // Only validate the password here if the input has previously been validated (login attempted but validation failed)
        const error = previousState ? !this.validatePassword(password) : false;
        // Dispatch email updates to the store
        this.store.dispatch(new globalHeaderAccountActions.ChangedPassword(password, error));
    }

    validatePassword(password) {
        // Helper method for checking that the latest entry for password is valid
        // Check that the password length is greater than 0
        return password && password.trim().length > 0;
    }

    handlePasswordVisibilityToggle(event) {
        const status = this.revealPassword ? false : true;
        // Handle password visibility button click
        // If revealPassword is currently set to true, set to opposite (false), for example
        this.store.dispatch(new globalHeaderAccountActions.ChangePasswordVisibility(status));
    }

    handleLogin(event) {
        // Wrapper for calling the login process
        // Validate the email and password, then actually begin login process if both are valid
        // Dependencies: validateLoginCredentials(), initiateLogin()

        // Retrieve the necessary form data
        const emailAddress = this.emailAddress;
        const password = this.password;
        const rememberMe = this.rememberMe;
        const formData = {
          emailAddress,
          password,
          rememberMe
        };

        // Reset the error states prior to validating the credentials
        const emailError = false;
        const passwordError = false;
        this.store.dispatch(new globalHeaderAccountActions.ResetLoginErrors(emailError, passwordError));

        // Validate the login credentials
        // Initiate the login process if the validation is successful
        if (this.validateLoginCredentials(formData)) {
            this.initiateLogin(formData);
        }

    }

    initiateLogin(formData) {
        // Actual login process
        // Initates effects that log user in
        // Construct the specific header required by login API
        const headers = {
            'Content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'X-Requested-With': 'XMLHttpRequest'
        };
        // Construct the necessary payload, mapping the form data to the specific keys required by the login API
        const payload = {
            'resetPassword.redirect': this.resetPasswordRedirect,
            'login.emailAddress': formData.emailAddress,
            'login.password': formData.password
        };

        // Add the remember me parameter to the payload ONLY if it is present
        if (formData.rememberMe) {
            payload['form.rememberMe'] = formData.rememberMe;
        }

        // Get language for requests
        const lang = this.langService.getLanguage();
        // Initiate the login process effect
        this.store.dispatch(new globalHeaderAccountActions.InitiateLoginProcess(payload, headers, lang));
    }

    handleLogout(event) {
        // Wrapper for calling the logout process
        // Dependencies: initiateLogout()
        this.initiateLogout();
    }

    initiateLogout() {
        // Actually begins the logout process
        // Using simple vanilla http request as there are no UI side effects needed on logout
        // Construct the specific header required by login API
        const headers = {
            'Content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'X-Requested-With': 'XMLHttpRequest'
        };
        // Get language for requests
        const lang = this.langService.getLanguage();
        // Initiate the logout process effect
        this.store.dispatch(new globalHeaderAccountActions.InitiateLogoutProcess(headers, lang));
    }

    validateLoginCredentials({ emailAddress, password}) {
        // Check that the email and password are valid in one operation
        // -(provide quick way to ensure that both are checked during login process)
        // Perform validation process on email and password
        const emailError = !this.validateEmailAddress(emailAddress);
        const passwordError = !this.validatePassword(password);
        // Dispatch status of validations as well as indication validation has occured (separate flag)
        const hasValidationCompleted = true;
        this.store.dispatch(new globalHeaderAccountActions.ValidatedLoginCredentials(emailError, passwordError, hasValidationCompleted));
        // Return false if any of the validations result in an error state
        return !emailError && !passwordError;
    }

    handleRememberMe(event) {
        const checked = event.checked;
        // Update the remember me state
        this.store.dispatch(new globalHeaderAccountActions.RememberMeToggle(checked));
    }

    myAccountClose() {
        // Tell global header window manager to close the My Account flyout
        this.closeMyAccount.emit({'close': true});
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

}
