import { Component, OnInit, OnDestroy } from '@angular/core';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { distinctUntilChanged, filter, map, tap } from 'rxjs/operators';
import { select } from '@ngrx/store';

// reducers
import * as fromApp from '../../../../../reducers';
import * as globalHeaderLocalizeActions from '../../../actions/global-header/global-header-localize/global-header-localize.action';

// models
import { GlobalHeaderLocalize } from '../../../reducers/global-header-localize/global-header-localize.reducer';
import { GlobalHeaderLocalizeAkamai, GlobalHeaderLocalizeUI } from '../../../models/global-header-localize.interface';
import { PickUpStoreCookie } from '../../../../../models/product-localized-data.interface';

// services
import { DialogService } from '@hdca/fe-angular-dialog';
import { CookieService } from 'ngx-cookie';
import { GeoLocatorService } from '../../../../../services/geo-locator/geo-locator.service';
import { Subscription } from 'rxjs';

import { ManageSubscriptionsContainer, StoreConstants, UserInfoService, LanguageService } from '@hdca/fe-angular-shared';
import { PickUpStoreIdReceived } from '../../../actions/global-header/global-header-localize/global-header-localize.action';
import { getGlobalHeaderLocalize } from '../../../../../reducers';

@Component({
  selector: 'global-header-localization',
  templateUrl: './global-header-localization.container.html'
})
export class GlobalHeaderLocalizationContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  localize$: Observable<GlobalHeaderLocalize>;
  pickUpStoreIdSubscription: Subscription;
  localizeState: GlobalHeaderLocalize;
  userId: string;
  directions: string;
  isMobileView = false;
  maxMobilePx = 767;

  constructor(
    private store: Store<fromApp.State>,
    private langService: LanguageService,
    private dialog: DialogService,
    private cookieService: CookieService,
    private user: UserInfoService,
    private geoLocatorService: GeoLocatorService) {
    super();
  }

  ngOnInit() {
    const currentUser = this.user.getUserAccountData();
    this.userId = currentUser.email || 'anonymous';

    this.getStores(); // Get observables
    this.pushSubscriptions(); // Perform Subscriptions
    this.checkIfLocalized(); // If not already localized, will perform autoLocalizationProcess()
    this.pickUpStoreIdSubscription = this.initPickUpStoreId$();
    if (typeof window !== 'undefined' && window.innerWidth <= this.maxMobilePx) {
      this.isMobileView = true;
    }
  }

  ngOnDestroy() {
    super.ngOnDestroy();
    this.pickUpStoreIdSubscription.unsubscribe();
  }

  getStores() {
    this.localize$ = this.store.select(state => state.globalHeaderLocalize);
  }

  pushSubscriptions() {
    const localizeSub: Subscription = this.localize$.subscribe({
      next: state => {
        this.localizeState = state;
        this.directions = `https://www.bing.com/maps?rtp=~pos.${state.latitude}_${state.longitude}_`;
      }
    });
    this.subscriptions.push(localizeSub);
  }

  /******************************************
   // SETUP
   ******************************************/

  checkIfLocalized(): void {
    const storeId: string = (typeof document !== 'undefined') ? this.cookieService.get(StoreConstants.STORE_KEY) : null;
    if (!storeId) {
      this.autoLocalizationProcess();
    } else {
      const params = {
        pageSize: 1,
        currentPage: 0,
        query: storeId
      };
      // Avoid service calls for localization if localized store exists
      if (this.localizeState.storeId !== storeId) {
        this.store.dispatch(new globalHeaderLocalizeActions.LoadLocalize(this.langService.getLanguage(), params, this.userId));
      }
    }
  }

  autoLocalizationProcess() {
    const localizeParams = {
      pageSize: '1',
      currentPage: '0',
      updateMyStore: true
    };
    this.store.dispatch(new globalHeaderLocalizeActions.LoadAutoLocalize(this.langService.getLanguage(), localizeParams, this.userId, true));
  }

  initPickUpStoreId$() {
    return this.store.pipe(
      select(getGlobalHeaderLocalize),
      filter(localizeState => {
        return localizeState.localized !== undefined;
      }),
      map(localizeState => localizeState.storeId),
      distinctUntilChanged(),
      tap(storeId => {
        if (!storeId) {

          // Setting up some variables to help validate the string from the object
          const pickUpStoreCookieString: string = typeof document !== 'undefined' ? (this.cookieService.get('pickupStore') || this.cookieService.get('store')) : '';

          if (!!pickUpStoreCookieString) {

            const pickUpStoreCookie: PickUpStoreCookie = JSON.parse(pickUpStoreCookieString);

            const pickUpStoreCookieId = pickUpStoreCookie.storeID + '';
            const regex = new RegExp('[0-9]{4}');

            // Only store this id if it exists and only if it contains 4 numerical characters
            if (!!pickUpStoreCookieId && (regex.test(pickUpStoreCookieId) && pickUpStoreCookieId.length === 4)) {
              this.store.dispatch(new PickUpStoreIdReceived(pickUpStoreCookieId));
            }

          } else {
            this.store.dispatch(new PickUpStoreIdReceived('9999'));
          }

        } else {
          this.store.dispatch(new PickUpStoreIdReceived(storeId));
        }
      }),
    ).subscribe();
  }

  /******************************************
   // EVENTS
   ******************************************/

  onSelectAStoreClick() {
    const pageSize = '3',
      currentPage = '0',
      query = '',
      latitude: number = this.localizeState.latitude,
      longitude: number = this.localizeState.longitude,
      lang: string = this.langService.getLanguage();

    this.store.dispatch(new globalHeaderLocalizeActions.LoadCommerceStoreList(pageSize, currentPage, query, lang, latitude, longitude));
    this.dialog.open('dialog-store-selector');
  }

  onAutoLocateClick() {
    const config: GlobalHeaderLocalizeAkamai = {
      autoLocateLoading: true
    };
    this.store.dispatch(new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeAkamai(config));

    // GET CURRENT LOCATION USING BROWSER
    const locateSub: Subscription = this.geoLocatorService.getLocation().subscribe(
      rep => {
        const params = {
          pageSize: 1,
          currentPage: 0,
          latitude: rep.coords.latitude,
          longitude: rep.coords.longitude
        };
        this.store.dispatch(new globalHeaderLocalizeActions.LoadLocalize(this.langService.getLanguage(), params, this.userId));
        config.autoLocateLoading = false;
        this.store.dispatch(new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeAkamai(config));
      }
    );

    this.subscriptions.push(locateSub);
  }

  onHoursClick() {
    const newState: GlobalHeaderLocalizeUI = {
      hours: !this.localizeState.ui.hours
    };
    this.store.dispatch(new globalHeaderLocalizeActions.ToggleGlobalHeaderLocalizeUI(newState));
  }

  onAkamaiCloseClick() {
    const newState: GlobalHeaderLocalizeUI = {
      akamai: false
    };
    this.store.dispatch(new globalHeaderLocalizeActions.ToggleGlobalHeaderLocalizeUI(newState));
  }

}
