// Angular core imports
import { Component, OnInit, OnDestroy } from '@angular/core';
// ngrx store imports
import { Store } from '@ngrx/store';
import * as fromApp from '../../../../../reducers';
import { GlobalHeaderSearchState } from '../../../reducers/global-header-search/global-header-search.reducer';
import * as searchActions from '../../../actions/global-header/global-header-search/global-search.action';
import { Subscription } from 'rxjs';
// Shared app imports
import { LanguageService, StoreService, ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
// Subscription importsef
import { TranslateService } from '@ngx-translate/core';
import { Observable } from 'rxjs';

@Component({
  selector: 'global-header-search-input-field',
  templateUrl: './global-header-search-input-field.container.html'
})
export class GlobalHeaderSearchInputFieldContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  currentSearchInput: string;
  keyword: String;
  searchTerm: string;
  searchGlobalListenFunction: Function;
  hideAutoSuggest: boolean;
  query: string;
  showDetails = false;
  inputPlaceholder: string;
  searchState: Observable<GlobalHeaderSearchState>;
  typingTimeout: any = null;

  hideAutoSuggestResultsAndResetField() {
    // Hide the autosuggest results and reset search field if search field reset is pressed
    this.store.dispatch(new searchActions.HideAutoSuggest(true));
    this.currentSearchInput = '';
  }

  onChange(value) {
    clearTimeout(this.typingTimeout);
    const transformedQuery = this.transformQuery(value);
    // Get language for requests
    const lang = this.langService.getLanguage();
    // Clear all results whenever the search term has changed, to prepare for new results and trigger the loading states
    this.store.dispatch(new searchActions.ClearSearchKeywords('Online', lang, transformedQuery));
    this.store.dispatch(new searchActions.ClearTopSearchResults('Online', lang, transformedQuery));
    // Only dispatch change events if the text in the field is not the same as the last search
    if (value.length > 2 && value !== this.searchTerm) {
      // Initiate search if greater than two characters and populate autosuggest combobox
      // Dispatch calls to update search keywords and top results
      this.store.dispatch(new searchActions.HideAutoSuggest(false));
      // set timeout to avoid overloading requests when fast typing
      this.typingTimeout = setTimeout(() => {
        this.store.dispatch(new searchActions.LoadSearchKeywords('Online', lang, transformedQuery));
      }, 500);
      this.store.dispatch(new searchActions.SearchTermChanged(value));
    } else {
      // Set the HideAutoSuggest to false, causing the ng-container block containing autosuggest to remove said container (no render)
      this.store.dispatch(new searchActions.SearchTermChanged(value));
      this.store.dispatch(new searchActions.HideAutoSuggestRetainSearchTerm(true));
    }
  }

  handleSearchButtonPress(value) {
    // Redirect user to search results page if clicking the search button (instead of clicking a keyword in the autosuggest)
    const selectedKeyword = this.searchTerm ? encodeURIComponent(this.searchTerm.trim()) : this.transformQuery(value) ? encodeURIComponent(this.transformQuery(value).trim()) : '';
    this.store.dispatch(new searchActions.LoadSearchResults(this.langService.getLanguage(), selectedKeyword, this.storeService.getStore()));
  }

  ngOnInit() {
    const translateSub = this.translate.get('GLOBAL_HEADER.SEARCH.SEARCH_PLACEHOLDER').subscribe(placeholder => {
      this.inputPlaceholder = placeholder;
    });
    // Subscribe to the global header search state
    // User navigating using the keyboard will change the current search term to reflect the search keyword currently being highlighted,
    // need to update search input field value to reflect that
    this.searchState = this.store.select(fromApp.getGlobalHeaderSearch);
    const searchInputSub: Subscription = this.searchState.
      subscribe((state: GlobalHeaderSearchState) => {
        this.keyword = state.keyword;
        this.hideAutoSuggest = state.hideAutoSuggest;
        this.searchTerm = state.searchTerm;
        this.query = state.query;
        this.showDetails = state.showDetails;
      });
    this.subscriptions.push(searchInputSub, translateSub);
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  transformQuery(query: string): string {
    // strip special characters
    let transformedQuery = query.replace(/:|,/g, ' ');
    // strip extra whitespace
    transformedQuery = transformedQuery.replace(/\s/g, ' ').split(' ').filter(el => el).join(' ');
    return transformedQuery;
  }

  onFocus(event) {
    this.inputPlaceholder = '';
  }

  onBlur(event) {
    this.inputPlaceholder = this.translate.instant('GLOBAL_HEADER.SEARCH.SEARCH_PLACEHOLDER');
  }

  constructor(
    private store: Store<fromApp.State>,
    private langService: LanguageService,
    private storeService: StoreService,
    private readonly translate: TranslateService
  ) { super(); }
}
