// core
import { Component, OnInit, OnDestroy, ChangeDetectorRef } from '@angular/core';
import { select, Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { Subscription } from 'rxjs';

// ngrx store
import * as fromApp from '../../../../reducers';
import * as fromGlobalHeaderOverall from '../../reducers/global-header-overall/global-header-overall.reducer';
import * as globalHeaderActions from '../../actions/global-header/global-header-overall/global-header-overall.action';

// services
import { getNodeByUid, LanguageService, UserInfoService,
  ManageSubscriptionsContainer, MY_ACCOUNT_NAV_NODE } from '@hdca/fe-angular-shared';

// models
import { GlobalHeaderLocalize } from '../../reducers/global-header-localize/global-header-localize.reducer';
import { NavNodes } from '../../models/nav-nodes.interface';

// Get constants for Nav Node keys
import {  tap } from 'rxjs/operators';
import * as navNodeActions from '../../actions/global-header/nav-nodes/nav-nodes.action';

@Component({
  selector: 'global-header-container',
  templateUrl: './global-header.container.html'
})
export class GlobalHeaderContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  lang: string;
  localize$: Observable<GlobalHeaderLocalize>;
  localizeState: GlobalHeaderLocalize;
  // Declare all UI visibility flags
  storeLocatorOpen: boolean;
  autoSuggestOpen: boolean;
  shopByDepartmentOpen: boolean;
  storeInfoOpen: boolean;
  myAccountOpen: boolean;
  flyouts: Array<string>;
  // UI text elements
  firstName: string;
  isAuth: boolean;
  isExpired: boolean;
  cartQuantity: string;
  // Navigation elements
  accountNavNodes?: NavNodes;
  navNodes$: Observable<NavNodes>;
  flyoutState: boolean;
  userId: string;
  isClient: boolean = (typeof window !== 'undefined');

  constructor(private store: Store<fromApp.State>,
              private langService: LanguageService,
              private ref: ChangeDetectorRef,
              private user: UserInfoService) {
    super();
  }

  ngOnInit() {
    // Get user name
    const currentUser = this.user.getUserAccountData();
    this.userId = currentUser.email || 'anonymous';

    this.lang = this.langService.getLanguage();
    this.getStores();
    this.getNavNodes();
    this.pushSubscriptions();

    // Set one time UI text elements
    this.firstName = currentUser.givenName ? currentUser.givenName[0].toUpperCase() + currentUser.givenName.slice(1) : '';
    this.isAuth = currentUser.isAuth;
    this.isExpired = currentUser.isExpired;

    // Get the current cart quantity
    const initialCartQuantity = currentUser.cartQuantity;
    this.store.dispatch(new globalHeaderActions.UpdateCartCount(initialCartQuantity));
  }

  getStores() {
    this.localize$ = this.store.select(state => state.globalHeaderLocalize);
  }

  getNavNodes() {
    this.navNodes$ = this.store.pipe(
      select(fromApp.getNavNodes),
      tap(navNodes => {
        if (!navNodes.client) {
          this.store.dispatch(new navNodeActions.LoadNavNodes(this.lang));
        }
      })
    );
  }

  pushSubscriptions() {
    const localizeSub: Subscription = this.localize$.subscribe({
      next: state => this.localizeState = state
    });

    // Manage the Header UI
    // - Goal: prevent conflicts between the various header components as some components affects the visibility of others
    const globalHeaderUI: Subscription = this.store.select(fromApp.getGlobalHeaderOverall).
    subscribe((state: fromGlobalHeaderOverall.GlobalHeaderOverallState) => {
      // Visibility controls from state
      this.storeLocatorOpen = state.storeLocatorOpen;
      this.autoSuggestOpen = state.autoSuggestOpen;
      this.shopByDepartmentOpen = state.shopByDepartmentOpen;
      this.storeInfoOpen = state.storeInfoOpen;
      this.myAccountOpen = state.myAccountOpen;
      const flyoutStatuses = {
        storeLocatorOpen: state.storeLocatorOpen,
        autoSuggestOpen: state.autoSuggestOpen,
        shopByDepartmentOpen: state.shopByDepartmentOpen,
        storeInfoOpen: state.storeInfoOpen,
        myAccountOpen: state.myAccountOpen
      };
      // Cart Quantity from state
      this.cartQuantity = state.cartQuantity;
      // Generate a list of all the UI components that are allowed to be visible according to the UX specification
      this.flyouts = Object.keys(flyoutStatuses).filter((headerComponent) => flyoutStatuses[`${headerComponent}`] === true);
      if (!this.ref['destroyed']) {
        this.ref.detectChanges();
      }
    });

    // Get Nav Node data (only if not available in page)
    const navNodeSub: Subscription = this.navNodes$.subscribe({
      next: state => {
        this.accountNavNodes = this.getAccountNavNode(state);

        if (!this.ref['destroyed']) {
          this.ref.detectChanges();
        }
      }
    });

    this.subscriptions.push(globalHeaderUI, navNodeSub, localizeSub);
  }

  getAccountNavNode(state) {
    // Goes through the main navigation object and gets the My Account links
    return getNodeByUid(state, MY_ACCOUNT_NAV_NODE);
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  /******************************************
   // EVENT RESPONSES
   ******************************************/

  // Control the appearance of the My Account feature
  openMyAccount() {
    // Open the My Account flyout
    this.store.dispatch(new globalHeaderActions.OpenGlobalHeaderMyAccount());
  }

  closeMyAccount() {
    // Close the My Account flyout
    this.store.dispatch(new globalHeaderActions.CloseGlobalHeaderMyAccount());
  }

  openFlyout() {
    this.store.dispatch(new globalHeaderActions.OpenGlobalHeaderFlyout());
  }

  closeFlyout() {
    this.store.dispatch(new globalHeaderActions.CloseGlobalHeaderFlyout());
  }

}
