// core
import { Component, OnInit, OnDestroy, Input } from '@angular/core';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { TranslateService } from '@ngx-translate/core';

// ngrx store
import * as fromApp from '../../../../reducers';
import * as globalHeaderLocalizeActions from '../../actions/global-header/global-header-localize/global-header-localize.action';

// services
import { LanguageService } from '@hdca/fe-angular-shared';
import { DialogService } from '@hdca/fe-angular-dialog';
import { UserInfoService } from '@hdca/fe-angular-shared';
import { GeoLocatorService } from '../../../../services/geo-locator/geo-locator.service';

// models
import { GlobalHeaderLocalize } from '../../reducers/global-header-localize/global-header-localize.reducer';
import { Alert, AlertType } from '../../../cart/models/alert.interface';

// Shared app imports
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
import { Subscription } from 'rxjs';

@Component({
  selector: 'store-selector-wrapper',
  templateUrl: './store-selector-wrapper.container.html'
})
export class StoreSelectorWrapperContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  alerts: Array<Alert> = [];
  langService: LanguageService;
  localize$: Observable<GlobalHeaderLocalize>;
  localizeState: GlobalHeaderLocalize;
  flyoutState: boolean;
  userId: string;
  @Input() presetDefaultQuery?: string;
  spinner = false;
  storeSelectorTitle: string;

  constructor(private store: Store<fromApp.State>,
              langService: LanguageService,
              private user: UserInfoService,
              private dialog: DialogService,
              private geoLocatorService: GeoLocatorService,
              private readonly translate: TranslateService) {
    super();
    this.langService = langService;
  }

  ngOnInit() {
    // Get user name
    const currentUser = this.user.getUserAccountData();
    this.userId = currentUser.email || 'anonymous';

    const lang = this.langService.getLanguage();
    this.getStores();
    this.pushSubscriptions(lang);
    this._getStoreSelectorTitle();

  }

  getStores() {
    this.localize$ = this.store.select(state => state.globalHeaderLocalize);
  }

  pushSubscriptions(lang) {
    const localizeSub: Subscription = this.localize$.subscribe({
      next: state => this.localizeState = state
    });
    this.subscriptions.push(localizeSub);
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  onStoreSelectorSearchClick(postal) {
    const pageSize = '3',
      currentPage = '0',
      query = postal.value,
      lang: string = this.langService.getLanguage(),
      latitude: number = this.localizeState.latitude,
      longitude: number = this.localizeState.longitude;
    this.store.dispatch(new globalHeaderLocalizeActions.LoadCommerceStoreList(pageSize, currentPage, query, lang, latitude, longitude, false));
  }

  onStoreSelectorNextClick() {
    this.onStoreSelectorClick(1);
  }

  onStoreSelectorPreviousClick() {
    this.onStoreSelectorClick(-1);
  }

  onStoreSelectorClick(direction: number) {
    const pageSize = '3',
      currentPage = `${this.localizeState.storeSelector.pagination.currentPage + direction}`,
      query = this.localizeState.storeSelector.query,
      lang: string = this.langService.getLanguage(),
      latitude: number = this.localizeState.latitude,
      longitude: number = this.localizeState.longitude;
    this.store.dispatch(new globalHeaderLocalizeActions.LoadCommerceStoreList(pageSize, currentPage, query, lang, latitude, longitude));
  }

  onStoreSelectorSelectClick(storeId) {
    const params = {
      pageSize: 1,
      currentPage: 0,
      query: storeId
    };
    this.store.dispatch(new globalHeaderLocalizeActions.LoadLocalize(this.langService.getLanguage(), params, this.userId));
    this.dialog.close('dialog-store-selector');
  }

  onAutoLocateClick() {
    const locateSub: Subscription = this.geoLocatorService.getLocation().subscribe(
      rep => {
        this.spinner = true;

        if (typeof window !== 'undefined') {
          setTimeout(() => {
            const params = {
              pageSize: '1',
              currentPage: '0',
              latitude: rep.coords.latitude,
              longitude: rep.coords.longitude
            };

            this.store.dispatch(new globalHeaderLocalizeActions.LoadAutoLocalize(this.langService.getLanguage(), params, this.userId, false));
            this.spinner = false;
            this.closeSelectMyStoreModal();
          }, 2000);
        }
      }, err => {
        const alert = [{
          id: 'geoError',
          type: AlertType.Warning,
          message: this.translate.instant('GLOBAL_HEADER.STORE_SELECTOR.SELECT_MY_STORE_GEO_ERROR'),
          messageList: [],
          dismissible: true
        }];

        this.alerts = alert;
        this.spinner = false;
      }
    );
    this.subscriptions.push(locateSub);
  }

  clearAlert(id) {
    this.alerts = this.alerts.filter(e => e.id !== id);
  }

  closeDialog() {
    this.alerts = [];
  }

  closeSelectMyStoreModal() {
    this.dialog.close('dialog-store-selector');
  }

  _getStoreSelectorTitle() {
    this.storeSelectorTitle = this.translate.instant('GLOBAL_HEADER.STORE_SELECTOR.SELECT_MY_STORE');
  }

}
