import { Directive, HostListener, OnInit, OnDestroy, ElementRef } from '@angular/core';
// ngrx store imports
import { Store } from '@ngrx/store';
import * as fromApp from '../../../../reducers';
import * as searchActions from '../../actions/global-header/global-header-search/global-search.action';
import { Subscription } from 'rxjs';
import * as fromGlobalHeaderSearch from '../../reducers/global-header-search/global-header-search.reducer';
// Model Imports
import { SearchKeywordSuggestions } from '../../models/global-header-search.interface';
// App imports
import { LanguageService, ManageSubscriptionsContainer, StoreService } from '@hdca/fe-angular-shared';
// Other app imports
import { CookieService } from 'ngx-cookie';

@Directive({
    selector: '[global-search-interaction]',
})
export class GlobalSearchDirective extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
    // Directive for listening to keyboard events while the autosuggest is open
    // There are three scenarios that need to be listened to:
    // - Pressing ENTER to initiate a search
    // - Pressing ESC to close the autosuggest
    // - Pressing UP or DOWN to scroll through the keyword suggestions
    // Listening to the keydown event on window (window:keydown)
    // - Target: window
    // - Event name: keydown
    selectedIndex: number;
    keywordsuggestions: SearchKeywordSuggestions;
    query: string;
    searchTerm: string;
    hideAutoSuggest: boolean;

    @HostListener('window:click', ['$event'])
    onClick($event: UIEvent) {
        if (!this.el.nativeElement.contains($event.target)) {
            // Hide the autosuggest if someone clicks outside of it
            this.store.dispatch(new searchActions.HideAutoSuggestRetainSearchTerm(true));
        }
    }
    // Listen for window keydown so that keyboard navigation can be done with search keywords
    // Keyboard also needed to submit search and close autosuggest
    @HostListener('window:keydown', ['$event', '$event.keyCode'])
    onKeyDown($event: KeyboardEvent, keyCode) {

        const keyCodes = {
            ENTER_KEY: 13,
            ESCAPE: 27,
            ARROW_DOWN: 40,
            ARROW_UP: 38,
        };

        // Submit the search if user presses enter
        if (keyCode === keyCodes.ENTER_KEY && (!this.hideAutoSuggest || ($event && $event.target && $event.target['nodeName'] === 'INPUT'))) {
            if (typeof window !== 'undefined') {
                // Get selected keyword
                const index = this.selectedIndex;
                const suggestions = this.keywordsuggestions;
                const searchTerm = this.transformQuery(this.searchTerm);
                const selectedKeyword = index !== -1 ? encodeURIComponent(suggestions[index]) : encodeURIComponent(searchTerm);
                // Perform redirect
                if (selectedKeyword) {
                    this.store.dispatch(new searchActions.LoadSearchResults(this.langService.getLanguage(), selectedKeyword, this.storeService.getStore()));
                }
            }
        }

        // Use escape key to close the autosuggest
        if (keyCode === keyCodes.ESCAPE) {
            // Clear the entire autosuggest state complex, causing it to be removed
            this.removeAutosuggest();
        }

        // Short circuit the callback if there is no autosuggest visible
        // Don't need to fire keyboard navigation if that's the case
        if (this.query.length < 3) {
            return;
        }

        if ((keyCode === keyCodes.ARROW_UP || keyCode === keyCodes.ARROW_DOWN) && !this.hideAutoSuggest) {
            // Handle browsing the keyword suggestions using up and down arrows
            const query = this.query;
            const selectedIndex = this.selectedIndex;

            // Blur the input field if the user is navigating by arrow key
            // Focus on the input field will clash with keyboard navigation if it is allowed to remain
            const target = $event.target as HTMLInputElement;
            target.blur();

            // stop entire screen from scrolling when there are suggestions
            // Fix originally implemented in SCI-140 for React header
            $event.preventDefault();

            // Only fire off state updates if there is text in the input field
            if (query.length > 0) {
                // Get suggestion list
                const suggestions = this.keywordsuggestions;

                // Additional sanity check, make sure there are suggestions before updating keyword selected index
                if (suggestions) {
                    // Find max index of suggestions list
                    const maxIndex = suggestions.length - 1;

                    if (keyCode === keyCodes.ARROW_UP) {
                        // If going up, rollback if at top (to last keyword)
                        if (selectedIndex === -1) {
                            this.changeSelectedIndex(maxIndex, true);
                        } else {
                            this.changeSelectedIndex(selectedIndex - 1, true);
                        }
                    }
                    if (keyCode === keyCodes.ARROW_DOWN) {
                        // If going down, rollover if at bottom (to first keyword)
                        if (selectedIndex === maxIndex) {
                            this.changeSelectedIndex(-1, true);
                        } else {
                            this.changeSelectedIndex(selectedIndex + 1, true);
                        }
                    }
                }

            }
        }
    }

    transformQuery(query: string): string {
        // strip special characters
        let transformedQuery = query.replace(/:|,/g, ' ');
        // strip extra whitespace
        transformedQuery = transformedQuery.replace(/\s/g, ' ').split(' ').filter(el => el).join(' ');
        return transformedQuery;
    }

    removeAutosuggest() {
        // Use the hideAutoSuggest state to control appearance of autosuggest combobox
        // Reset autosuggest data since it is no longer needed
        // - Any change to search input will override this content so no need to retain it
        // - Need to ensure loading stencils appear again properly
        this.store.dispatch(new searchActions.HideAutoSuggestRetainSearchTerm(true));
    }

    changeSelectedIndex(index, arrowKey = false) {
        // Update the selected index and the selected keyword on keystrokes
        if (index !== this.selectedIndex) {
            // Execute updates if the selected index has changed
            // Get the suggestions to determine what the new keyword is
            const suggestions = this.keywordsuggestions;
            const suggestionsIndex = index;
            const query = this.query;
            const keyword = index !== -1 ? suggestions[suggestionsIndex] : query;
            // Get parameters for loading new top product results
            const lang = this.langService.getLanguage();
            const localizedPOS = (typeof document !== 'undefined') ? this.cookieService.get('store') ? this.cookieService.get('store') : null : null;
            // Similar to hovering over a keyword, perform a new Top Results query to refresh the top 3 products whenever user browses with arrow keys
            this.store.dispatch(new searchActions.SearchKeywordSelectionChanged(index, keyword, false));
            this.store.dispatch(new searchActions.LoadTopSearchResults('Online', lang, keyword, localizedPOS, true));
        }
    }

    ngOnInit() {
        // Keep up to date on selected index for keyword list, for updating Top Results
        const selectedIndexSub: Subscription = this.store.select(fromApp.getGlobalHeaderSearch).
            subscribe((state: fromGlobalHeaderSearch.GlobalHeaderSearchState) => {
                this.selectedIndex = state.selectedIndex;
                this.keywordsuggestions = state.keywordsuggestions;
                this.query              = state.query;
                this.searchTerm         = state.searchTerm;
                this.hideAutoSuggest    = state.hideAutoSuggest;
            });
        this.subscriptions.push(selectedIndexSub);
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

    constructor(
        private store: Store<fromApp.State>,
        private langService: LanguageService,
        private storeService: StoreService,
        private cookieService: CookieService,
        private el: ElementRef
    ) { super(); }

}
