import { Injectable } from '@angular/core';
import { Effect, Actions } from '@ngrx/effects';
import { Action, Store } from '@ngrx/store';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';

// Import data services
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { DataSender } from '../../../../services/data-sender/data-sender.service';
import { ToggleApiError } from '../../../../actions/errors';

// Import actions
import * as globalHeaderLocalizeActions from '../../actions/global-header/global-header-localize/global-header-localize.action';

// Import models
import { LocalizeData, CommerceStoreData, GlobalHeaderLocalizeAkamai, GlobalHeaderLocalizeHours, GlobalHeaderLocalizeStoreSelector, GlobalHeaderLocalizeUI } from '../../models/global-header-localize.interface';
import { GlobalHeaderLocalize } from '../../reducers/global-header-localize/global-header-localize.reducer';

// Import URL constants
import { apiUrls } from '../../../../constants/api.urls';

// Services
import { CookieService } from 'ngx-cookie';
import * as constant from '@hdca/fe-angular-shared';
import { StoreConstants } from '@hdca/fe-angular-shared';
import { OriginService } from '../../../../services/origin/origin.service';

// Operators
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/mergeMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/skip';
import 'rxjs/add/operator/takeUntil';

import { timeFormat } from '@hdca/fe-angular-shared';
import { State } from '../../../../reducers';
import { StoreInformation } from '../../models/global-header-store-information.interface';
const errorUrl = null;

@Injectable()
export class GlobalHeaderLocalizeEffect {
    constructor(
        private actions$: Actions,
        private service: DataFetcher,
        private cookieService: CookieService,
        private originService: OriginService,
        private postService: DataSender,
        private store: Store<State>
    ) { }

    @Effect()
    loadAutoLocalize$: Observable<Action> = this.actions$
        .ofType<globalHeaderLocalizeActions.LoadAutoLocalize>(globalHeaderLocalizeActions.LOAD_AUTO_LOCALIZE)
        .switchMap((action) => {
            return this.postService
                .post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_LOCALIZE(
                    action.userId,
                    action.params.pageSize,
                    action.params.currentPage,
                    action.params.query,
                    action.params.latitude,
                    action.params.longitude
                ), action.lang, action.params)
                .mergeMap((payload: LocalizeData) => {
                    if (payload && payload.stores) {
                        typeof document !== 'undefined' && this.cookieService.put(StoreConstants.STORE_KEY, payload.stores[0].name, { domain: constant.domainCookies.ANY_HDCA, path: constant.domainCookies.PATH });

                        const openingHour = payload.stores[0].openingHours.weekDayOpeningList[this.getCurrentDay()].openingTime.formattedHour;
                        const closingHour = payload.stores[0].openingHours.weekDayOpeningList[this.getCurrentDay()].closingTime.formattedHour;
                        const storeHours = this.setStoreHours(closingHour, openingHour);
                        const storeTitleAkamai = `${payload.stores[0].displayName} #${payload.stores[0].name}`;
                        const isAkamaiPopupEnabled = action.isAkamaiPopupEnabled;

                        const localize: GlobalHeaderLocalize = {
                            localized: true,
                            storeHours: storeHours,
                            storeTitle: payload.stores[0].displayName,
                            storeId: payload.stores[0].name,
                            latitude: payload.sourceLatitude,
                            longitude: payload.sourceLongitude
                        };

                        const akamai: GlobalHeaderLocalizeAkamai = {
                            storeTitle: storeTitleAkamai,
                            storeHours: storeHours,
                            storePhone: payload.stores[0].address.phone,
                            storeAddress: payload.stores[0].address.formattedAddress
                        };

                        const ui: GlobalHeaderLocalizeUI = {
                            akamai: isAkamaiPopupEnabled
                        };

                        return [
                            new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalize(localize),
                            new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeAkamai(akamai),
                            new globalHeaderLocalizeActions.ToggleGlobalHeaderLocalizeUI(ui),
                            new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeHours(this.formStoreHours(payload.stores[0].openingHours, payload.stores[0].address))
                        ];
                    } else {
                        const localize: GlobalHeaderLocalize = {
                            localized: false
                        };

                        return [
                            new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalize(localize),
                        ];
                    }
                })
                .catch(err => {
                    const localize: GlobalHeaderLocalize = {
                        localized: false
                    };

                    this.store.dispatch(new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalize(localize));

                    return of(new ToggleApiError(err, errorUrl, (`failed`)));
                });
        });

    @Effect()
    loadLocalize$: Observable<Action> = this.actions$
        .ofType<globalHeaderLocalizeActions.LoadLocalize>(globalHeaderLocalizeActions.LOAD_LOCALIZE)
        .switchMap((action) => {
            return this.service
                .fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_STORE_INFORMATION(action.params.query), '')
                .mergeMap((payload: StoreInformation) => {
                    typeof document !== 'undefined' && this.cookieService.put(StoreConstants.STORE_KEY, payload.name, { domain: constant.domainCookies.ANY_HDCA, path: constant.domainCookies.PATH });

                    const openingHour = payload.openingHours.weekDayOpeningList[this.getCurrentDay()].openingTime.formattedHour;
                    const closingHour = payload.openingHours.weekDayOpeningList[this.getCurrentDay()].closingTime.formattedHour;
                    const storeHours = this.setStoreHours(closingHour, openingHour);

                    const localize: GlobalHeaderLocalize = {
                        localized: true,
                        storeHours: storeHours,
                        storeTitle: payload.displayName,
                        storeId: payload.name,
                        latitude: payload.geoPoint.latitude,
                        longitude: payload.geoPoint.longitude
                    };

                    return [
                        new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalize(localize),
                        new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeHours(this.formStoreHours(payload.openingHours, payload.address))
                    ];
                })
                .catch(err => of(new ToggleApiError(err, errorUrl,
                    (`failed`))));
        });

    @Effect()
    loadCommerceStore$: Observable<Action> = this.actions$
        .ofType<globalHeaderLocalizeActions.LoadCommerceStore>(globalHeaderLocalizeActions.LOAD_COMMERCE_STORE)
        .switchMap((action) => {
            return this.service
                .fetch(apiUrls.BASE_URL_HOMEDEPOT_CA(this.originService.getOrigin(), action.lang) + apiUrls.GET_COMMERCE_STORE(action.query), action.lang)
                .mergeMap((payload: CommerceStoreData) => {
                    return [
                        new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeHours(this.formStoreHours(payload.stores[0].openingHours, payload.stores[0].address))
                    ];
                })
                .catch(err => of(new ToggleApiError(err, errorUrl,
                    (`failed`))));
        });

    @Effect()
    loadCommerceStoreList$: Observable<Action> = this.actions$
        .ofType<globalHeaderLocalizeActions.LoadCommerceStoreList>(globalHeaderLocalizeActions.LOAD_COMMERCE_STORE_LIST)
        .switchMap((action) => {
            // if not autolocalized, default to Leaside
            const DEFAULT_LAT = 43.7;
            const DEFAULT_LONG = -79.4;
            return this.service
                .fetch(apiUrls.BASE_URL_HOMEDEPOT_CA(this.originService.getOrigin(), action.lang) + apiUrls.GET_COMMERCE_STORE_LIST(action.pageSize, action.currentPage, action.query, action.latitude || DEFAULT_LAT, action.longitude || DEFAULT_LONG), action.lang)
                .mergeMap((payload: CommerceStoreData) => {
                    const storeSelector: GlobalHeaderLocalizeStoreSelector = {
                        stores: payload.stores,
                        pagination: {
                            totalPages: payload.pagination.totalPages,
                            totalResults: payload.pagination.totalPages,
                            currentPage: payload.pagination.currentPage,
                            pageSize: payload.pagination.pageSize
                        },
                        query: action.query,
                        isInGlobalHeader: action.isInGlobalHeader
                    };
                    return [
                        new globalHeaderLocalizeActions.UpdateGlobalHeaderLocalizeStoreSelector(storeSelector)
                    ];
                })
                .catch(err => of(new ToggleApiError(err, errorUrl,
                    (`failed`))));
        });

    getCurrentDay() {
        return new Date().getDay();
    }

    setStoreHours(closingTime: string, openingTime: string) {
        return `${timeFormat(openingTime)} - ${timeFormat(closingTime)}`;
    }

    formStoreHours(openingHours, address) {
        const weekList = openingHours.weekDayOpeningList ? openingHours.weekDayOpeningList : [];
        const formattedHourList = [];
        formattedHourList[0] = {
            day: 'M-F',
            time: `${timeFormat(weekList[1].openingTime.formattedHour)} - ${timeFormat(weekList[1].closingTime.formattedHour)}`,
        };
        formattedHourList[1] = {
            day: 'Sat',
            time: `${timeFormat(weekList[6].openingTime.formattedHour)} - ${timeFormat(weekList[6].closingTime.formattedHour)}`,
        };
        formattedHourList[2] = {
            day: 'Sun',
            time: `${timeFormat(weekList[0].openingTime.formattedHour)} - ${timeFormat(weekList[0].closingTime.formattedHour)}`,
        };
        const hours: GlobalHeaderLocalizeHours = {
            formattedHours: formattedHourList,
            phone: address.phone
        };
        return hours;
    }
}
