
import { Injectable } from '@angular/core';
import { Effect, Actions } from '@ngrx/effects';
import { Action } from '@ngrx/store';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';
// Import data services
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
// Import actions
import * as search from '../../actions/global-header/global-header-search/global-search.action';
// Import models
import {
    SearchKeywordSuggestions,
    SearchAutosuggestDetails,
    SKUSearchResult
} from '../../models/global-header-search.interface';
// Import URL constants
import { apiUrls } from '../../../../constants/api.urls';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/skip';
import 'rxjs/add/operator/takeUntil';

import { OriginService } from '../../../../services/origin/origin.service';

const getRedirectUrl = (query: string, lang: string): string => {
    const url = lang === 'fr' ? '/fr/accueil/resultats.html' : '/en/home/search.html';
    return `${url}?q=${query}#!q=${query}`;
};

@Injectable()
export class GlobalSearchEffects {
    constructor(
        private actions$: Actions,
        private service: DataFetcher,
        private originService: OriginService
    ) { }

    @Effect()
    loadSearchKeywordSuggestions$: Observable<Action> = this.actions$
        .ofType<search.LoadSearchKeywords>(search.LOAD_SEARCH_KEYWORD_SUGGESTIONS)
        .switchMap((action) => {
            return this.service
                .fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_SEARCH_KEYWORD_SUGGESTIONS(action.catalogVersion, action.query), action.lang)
                .map((searchKeywords: SearchKeywordSuggestions) => new search.LoadSearchKeywordsSuccess(searchKeywords))
                .catch(err => of(new search.LoadSearchKeywordsError(err)));
        });
    // Two possible top result behaviours
    // - Initial load when getting new search term, need to refresh entire Top Results state
    // - Selecting new keyword in keyword list, only update the top products but leave the subcategory list alone
    @Effect()
    loadSearchTopResults$: Observable<Action> = this.actions$
        .ofType<search.LoadTopSearchResults>(search.LOAD_SEARCH_TOP_RESULTS)
        .switchMap((action) => {
            return this.service
                .fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_SEARCH_TOP_RESULTS(action.catalogVersion, action.query, action.localizedPOS), action.lang)
                .map((searchTopResults: SearchAutosuggestDetails) => new search.LoadTopSearchResultsSuccess(searchTopResults, action.update))
                .catch(err => of(new search.LoadTopSearchResultsError(err)));
        });
    @Effect()
    loadSearchResults$: Observable<Action> = this.actions$
        .ofType<search.LoadSearchResults>(search.LOAD_SEARCH_RESULTS)
        .switchMap((action) => {
            if (!action.query.match(/.*\d{10}.*/)) {
                return of(new search.SearchRedirect(getRedirectUrl(action.query, action.lang)));
            }
            const origin = this.originService.getOrigin();
            return this.service
                .fetch(apiUrls.BASE_URL(origin) + apiUrls.GET_SKU(action.query, action.localizedPOS), action.lang)
                .map((sku: SKUSearchResult) => {
                    const redirectUrl = (sku && sku.url) ? `${origin}${sku.url}?searchterm=${action.query}` : getRedirectUrl(action.query, action.lang);
                    return new search.SearchRedirect(redirectUrl);
                })
                .catch(() => of(new search.SearchRedirect(getRedirectUrl(action.query, action.lang))));
        });
}
