// Import actions
import * as globalHeaderAccountActions from '../../actions/global-header/global-header-account/global-header-account.action';
// Model imports
import { AuthenticationState } from '../../models/global-header-account.interface';
// Constant imports
import { AuthenticationResponseConstants } from '@hdca/fe-angular-shared';

export interface GlobalHeaderAccountState {
    emailAddress: string;
    password: string;
    rememberMe: boolean;
    emailError: boolean;
    passwordError: boolean;
    authenticationInfo?: AuthenticationState;
    hasValidationCompleted: boolean;
    revealPassword: boolean;
    loggingIn: boolean;
}

export const nullState: GlobalHeaderAccountState = {
    emailAddress: '',
    password: '',
    rememberMe: false,
    emailError: false,
    passwordError: false,
    authenticationInfo: null,
    hasValidationCompleted: false,
    revealPassword: false,
    loggingIn: false
};

export const initialState = ((): GlobalHeaderAccountState => {
    return nullState;
})();

export function globalHeaderAccountReducer(state = initialState, action: globalHeaderAccountActions.GlobalHeaderAccountActions): GlobalHeaderAccountState {
  switch (action.type) {
    case globalHeaderAccountActions.CHANGED_EMAIL_ADDRESS: {
        // Update email address and email error flag on email address change in UI
        return {
            ...state,
            emailAddress: action.emailAddress,
            emailError: action.emailError
        };
    }

    case globalHeaderAccountActions.REMEMBER_ME_CHECK: {
        // Set the email address and remember me properties in state based on retrieved user data
        return {
            ...state,
            emailAddress: action.savedEmailAddress ? action.savedEmailAddress : state.emailAddress ? state.emailAddress : '',
            rememberMe: !!action.savedEmailAddress
        };
    }

    case globalHeaderAccountActions.REMEMBER_ME_TOGGLE: {
        // Set the email address and remember me properties in state based on retrieved user data
        return {
            ...state,
            rememberMe: !!action.checked
        };
    }

    case globalHeaderAccountActions.CHANGED_PASSWORD: {
        // Update password and password error on password change in UI
        return {
            ...state,
            password: action.password,
            passwordError: action.passwordError
        };
    }

    case globalHeaderAccountActions.CHANGED_PASSWORD_VISIBILITY: {
        // Update password visibility on reveal password button click in UI
        return {
            ...state,
            revealPassword: action.passwordVisible
        };
    }

    case globalHeaderAccountActions.RESET_LOGIN_ERRORS: {
        // Reset the email and password error states in the UI
        return {
            ...state,
            emailError: action.emailError,
            passwordError: action.passwordError
        };
    }

    case globalHeaderAccountActions.VALIDATED_LOGIN_CREDENTIALS: {
        // Update password and password error on password change in UI
        return {
            ...state,
            emailError: action.emailError,
            passwordError: action.passwordError,
            hasValidationCompleted: action.hasValidationCompleted
        };
    }

    case globalHeaderAccountActions.INITIATE_LOGIN_PROCESS: {
        // Action that fires when the login process has been initiated
        // Perform any desired UI side effects here (ex. show progress)
        return {
            ...state,
            loggingIn: true
        };
    }

    case globalHeaderAccountActions.LOGIN_RESPONSE: {
        // Action that fires when the login returns a response (either login allowed or login error for some reason)
        return {
            ...state,
            loggingIn: false,
            authenticationInfo: {
                ...state.authenticationInfo,
                code: action.code,
                success: action.success
            }
        };
    }

    case globalHeaderAccountActions.LOGIN_ERROR: {
        // Action that fires when the LOGIN PROCESS ITSELF FAILS (no response)
        return {
            ...state,
            loggingIn: false,
            authenticationInfo: {
                ...state.authenticationInfo,
                code: AuthenticationResponseConstants.AUTHENTICATION_ERROR,
                success: false
            }
        };
    }

    case globalHeaderAccountActions.EXPIRED_USER: {
        // Action that fires when the user session is expired
        return {
            ...state,
            loggingIn: false,
            authenticationInfo: {
                ...state.authenticationInfo,
                code: AuthenticationResponseConstants.AUTHENTICATION_EXPIRED,
                success: false
            }
        };
    }

    case globalHeaderAccountActions.LOGOUT_ERROR: {
        // Action that fires when the LOGOUT PROCESS ITSELF FAILS (no response)
        return {
            ...state
        };
    }

    case globalHeaderAccountActions.LOGOUT_SUCCESS: {
        // Action that fires when the LOGOUT PROCESS is successful (returns a good response)
        return {
            ...state
        };
    }

    default: {
        return state;
    }
  }
}
