import {
    SearchAutosuggestDetails,
    SearchKeywordSuggestions
} from '../../models/global-header-search.interface';
import * as searchActions from '../../actions/global-header/global-header-search/global-search.action';

export interface GlobalHeaderSearchState {
    autosuggestDetails: SearchAutosuggestDetails;
    keywordsuggestions: SearchKeywordSuggestions;
    searchTerm: string;
    selectedIndex: number;
    keyword: string;
    hideAutoSuggest: boolean;
    query: string; // Preserves the original search query, for reference when keyword is changed
    showDetails: boolean; // state for the first search without hovering or keydown
}

export const nullState: GlobalHeaderSearchState = {
  autosuggestDetails: null,
  keywordsuggestions: null,
  searchTerm: '',
  selectedIndex: -1,
  keyword: '',
  hideAutoSuggest: true,
  query: '',
  showDetails: false
};

export const initialState = ((): GlobalHeaderSearchState => {
    return nullState;
})();

export function globalHeaderSearchReducer(state = initialState, action: searchActions.GlobalSearchActions): GlobalHeaderSearchState {
  switch (action.type) {
    case searchActions.LOAD_SEARCH_TOP_RESULTS: {
        if (action.update) {
            // If loading new top search results when hovering over different keywords, only top products update
            // - The top categories under the search term do not change unless the search term itself changes
            return {
                ...state,
                autosuggestDetails: {
                    ...state.autosuggestDetails,
                    topsearches: null
                },
                showDetails: true
            };
        } else {
            return {
                ...state,
                searchTerm: action.query,
                showDetails: true
            };
        }
    }

    case searchActions.LOAD_SEARCH_KEYWORD_SUGGESTIONS: {
        return {
            ...state,
            selectedIndex: -1,
            query: action.query,
            keyword: action.query,
            showDetails: false
        };
    }

    case searchActions.CLEAR_SEARCH_TOP_RESULTS: {
        return { ...state, autosuggestDetails: null };
    }

    case searchActions.CLEAR_SEARCH_KEYWORD_SUGGESTIONS: {
        return { ...state, keywordsuggestions: null };
    }

    case searchActions.SEARCH_TOP_RESULTS_SUCCESS: {
        // Load top results success reducer
        // - If there is no data returned, set the topsearches parameter to an empty array
        // so we can distinguish between loading and loaded states
        const autosuggestDetails = action.payload;
        const topsearches = autosuggestDetails['topsearches'];
        return {
            ...state,
            autosuggestDetails: {
                ...autosuggestDetails,
                topsearches: topsearches ? topsearches : []
            }
        };
    }

    case searchActions.SEARCH_TOP_RESULTS_ERROR: {
        // Set top results to empty array if there is an error and no data can be returned
        return {
            ...state,
            autosuggestDetails: {
                ...state.autosuggestDetails,
                topsearches: []
            }
        };
    }

    case searchActions.SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_SUCCESS: {
        // Load search keyword suggestions reducer
        // - If there is no data returned, set the keywordsuggestions parameter to an empty array
        // so we can distinguish between loading and loaded states
        const keywordsuggestions = action.payload['keywordsuggestions'];
        if (keywordsuggestions && keywordsuggestions.length > 0) {
            return {
                ...state,
                keywordsuggestions
            };
        } else {
            return {
                ...state,
                hideAutoSuggest: true,
                keywordsuggestions: []
            };
        }
    }

    case searchActions.SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_ERROR: {
        // Set keywords to empty array if there is an error and no data can be returned
        return {
            ...state,
            keywordsuggestions: []
        };
    }

    case searchActions.SEARCH_TERM_CHANGED: {
        const term = action.payload;
        return {
            ...state,
            searchTerm: term
        };
    }

    case searchActions.SEARCH_KEYWORD_SELECTION_CHANGED: {
        const index = action.selectedIndex;
        const keyword = action.keyword;
        // Check if it was a hover event or not
        // The search term should only be changed to match the keyword during keyboard navigation
        const hover = action.hover;
        if (hover) {
            return {
                ...state,
                selectedIndex: index,
                keyword: keyword
            };
        } else {
            return {
                ...state,
                selectedIndex: index,
                keyword: keyword,
                searchTerm: keyword
            };
        }
    }

    case searchActions.HIDE_AUTO_SUGGEST: {
        const hide = action.payload;
        // If hiding autosuggest, reset everything
        if (hide) {
            return {
                ...state,
                autosuggestDetails: null,
                keywordsuggestions: null,
                selectedIndex: -1,
                hideAutoSuggest: true,
                query: '',
                searchTerm: '',
                keyword: ''
            };
        } else {
            return  {
                ...state,
                hideAutoSuggest: hide
            };
        }
    }

    case searchActions.HIDE_AUTO_SUGGEST_RETAIN_SEARCH_TERM: {
        const hide = action.payload;
        // If hiding autosuggest based on keyboard input, retain searchTerm in input field (don't do complete reset and clear out field)
        if (hide) {
            return {
                ...state,
                autosuggestDetails: null,
                keywordsuggestions: null,
                selectedIndex: -1,
                hideAutoSuggest: true,
                keyword: ''
            };
        } else {
            return  {
                ...state,
                hideAutoSuggest: hide
            };
        }
    }

    case searchActions.SEARCH_REDIRECT:
        const redirectUrl = action.url;
        if (redirectUrl) {
            window.location.href = redirectUrl;
        }
        return state;

    default: {
        return state;
    }
  }
}
