import { Injectable, Inject, PLATFORM_ID } from '@angular/core';
import { DOCUMENT, isPlatformBrowser, isPlatformServer } from '@angular/common';
import { REQUEST } from '@nguniversal/express-engine/tokens';
import { TranslateService as NGXTranslateService } from '@ngx-translate/core';
import { Observable, of } from 'rxjs';
import { LanguageService } from '@hdca/fe-angular-shared';
import { ILang } from './translation.interface';

const LANG_LIST: ILang[] = [
  { 'code': 'fr', 'name': 'French', 'culture': 'fr-CA' },
  { 'code': 'en', 'name': 'English', 'culture': 'en-CA' }
];
const LANG_DEFAULT: ILang = LANG_LIST[1];
const STORAGE_LANG_NAME = 'langCode';

@Injectable()
export class TranslatesService {
  constructor(@Inject(PLATFORM_ID) private _platformId: Object,
              @Inject(DOCUMENT) private _document: any,
              @Inject(REQUEST) private _request: any,
              @Inject(NGXTranslateService) private _translate: NGXTranslateService,
              @Inject(LanguageService) private _langService: any
  ) { }

  public initLanguage(): Promise<any> {
    return new Promise((resolve: Function) => {
      this._translate.addLangs(LANG_LIST.map((lang: ILang) => lang.code));
      this._translate.setDefaultLang(LANG_DEFAULT.code);
      const language: ILang = this._getLanguage();
      this._setLanguage(language);
      resolve();
    });
  }

  private _getLanguage(): ILang {
    let language: ILang = this._langService.getLanguage();
    if (language) {
      return language;
    }
    if (isPlatformBrowser(this._platformId)) {
      language = this._getFindLang(this._translate.getBrowserLang());
    }
    if (isPlatformServer(this._platformId)) {
      try {
        const reqLangList: string[] = this._request.headers['accept-language'].split(';')[0].split(',');
        language = LANG_LIST.find((lang: ILang) => reqLangList.indexOf(lang.code) !== -1 || reqLangList.indexOf(lang.culture) !== -1);
      } catch (err) {
        language = LANG_DEFAULT;
      }
    }
    language = language || LANG_DEFAULT;
    this._langService.setLanguage(language.code);
    return language;
  }

  private _getFindLang(code: string): ILang | null {
    return code ? LANG_LIST.find((lang: ILang) => lang.code === code) : null;
  }

  private _setLanguage(lang: ILang): void {
    this._translate.use(lang.code).subscribe(() => {
      this._document.documentElement.lang = lang.code;
    });
  }

  public changeLang(code: string): void {
    const lang: ILang = this._getFindLang(code);
    if (!lang || lang.code === this._translate.currentLang) {
      return;
    }
    this._langService.setLanguage(lang.code);
    window.location.reload();
  }

  public getLangList(): Observable<ILang[]> {
    return of(LANG_LIST);
  }

  public getCurrentLang(): string {
    return this._translate.currentLang;
  }
}
