import { Injectable, Inject } from '@angular/core';
// http deps
import {
  HttpInterceptor,
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpResponse,
  HttpParams
} from '@angular/common/http';
import { Store } from '@ngrx/store';
import { State } from '../../reducers';

// rxjs deps
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/observable/of';
import 'rxjs/add/operator/do';
import 'rxjs/add/observable/throw';
import { of } from 'rxjs';
import { finalize, tap } from 'rxjs/operators';

// env settings and mock engine
import { hash, production } from '../../tokens';

// cookie service
import { CookieService } from 'ngx-cookie';

// constants
import { UserConstants } from '@hdca/fe-angular-shared';
import { URL_PATTERNS } from './patterns.config';

// event transceiver stuff
import { EventTransceiverReceiveHTTPReturnEvent } from '../../modules/event-transceiver/actions';
import { HttpReturnRecord } from '../../modules/event-transceiver/models/web-performance.interface';

// State Transfer
import { TransferState, makeStateKey } from '@angular/platform-browser';

@Injectable()
export class FEAHttpInterceptor implements HttpInterceptor {
  headerKey = 'apiCall';
  langKey = 'lang';
  versionKey = 'fe_version';
  fieldsKey = 'fields';
  basicFields = 'BASIC';
  httpCounter = 0;
  hash = '';
  production = false;
  constructor(private store: Store<State>, private cookieService:
    CookieService, private state: TransferState,
    @Inject(hash) hashDi,
    @Inject(production) productionDi
  ) {
    this.hash = hashDi;
    this.production = productionDi;
  }

  addDefaultHeaderForGetRequest(request: HttpRequest<any>, lang: string): HttpRequest<any> {
    const bearer = this.getAuthorization();
    // this is for PETA request
    if (request.url.includes('sourcing?sap-client') || request.url.includes('sourcing-dot-caschain-events-sandbox-thd')) {
      return request.clone({
        setHeaders: {
          'Content-type': 'application/json; charset=UTF-8',
          'Accept-Language': lang,
          'Accept': 'application/json, text/javascript, */*',
          'X-Requested-With': 'XMLHttpRequest'
        }
      });
    } else if (this.isOpenAPI(request.url) || !bearer) {
      return request.clone({
        setHeaders: {
          'Accept-Language': lang,
          'Accept': 'application/json, text/javascript, */*',
          'X-Requested-With': 'XMLHttpRequest'
        }
      });
    } else {
      return request.clone({
        setHeaders: {
          'Accept-Language': lang,
          'Accept': 'application/json, text/javascript, */*',
          'X-Requested-With': 'XMLHttpRequest',
          'Authorization': `Bearer ${bearer}`
        }
      });
    }
  }

  prepareForCache(request: HttpRequest<any>): HttpRequest<any> {
    let newParams = new HttpParams({ fromString: request.params.toString() });
    newParams = newParams.delete(this.headerKey);

    // This caches the assets coming from node server with one auto-generated version
    if (this.isDictionaryRequest(request.url)) {
      newParams = newParams.append(this.versionKey, this.hash);
    }
    // Basic fields for server side calls
    if (this.isSSRCall(request.url)) {
      newParams = newParams.append(this.fieldsKey, this.basicFields);
    }
    // the automatic cache busting is disabled for Hybris and AEM endpoints.
    return request.clone({
      params: newParams
    });
  }

  getAuthorization() {
    if (typeof document !== 'undefined') {
      return this.cookieService.get(UserConstants.UUID_KEY);
    } else {
      return '';
    }
  }

  isOpenAPI(url: string): boolean {
    return url.includes(URL_PATTERNS.SERVICE)
      || url.includes(URL_PATTERNS.SURVEY)
      || url.includes(URL_PATTERNS.CONFIG)
      || url.includes(URL_PATTERNS.PROREFERRAL)
      || url.includes(URL_PATTERNS.COMMERCE_TOKEN)
      || url.includes(URL_PATTERNS.LOGIN)
      || url.includes(URL_PATTERNS.LOGOUT)
      || url.includes(URL_PATTERNS.COMMERCE_STORE)
      || url.includes(URL_PATTERNS.NAVNODES);
  }

  isDictionaryRequest(url: string): boolean {
    return url.includes('en.json') || url.includes('fr.json');
  }

  isSSRCall(url: string): boolean {
    return url.includes(URL_PATTERNS.NAVNODES) && typeof window === 'undefined';
  }

  intercept(
    request: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    let newRequest: HttpRequest<any> = null;
    // if api call and not translation call
    if ((request.params.get(this.headerKey) || request.method === 'POST')) {
      const lang = request.params.get(this.langKey) || 'en';
      newRequest = this.addDefaultHeaderForGetRequest(request, lang);
      if (!this.production) {
        // dynamically hit mockdata engine for dev calls
        return of(new HttpResponse({ status: 200, body: require('./stub.engine').ConfigEngine(request.url) }));
      }
    }
    const result = this.prepareForCache(newRequest ? newRequest : request);
    typeof window === 'undefined' && console.log('Outgoing request --> ', result.urlWithParams);

    let status: string;
    const started = Date.now();
    const key = makeStateKey(`HTTP_CALL${this.httpCounter++}`);
    return next.handle(result).pipe(
      tap(
        // Succeeds when there is a response; ignore other events
        event => status = event instanceof HttpResponse ? 'succeeded' : '',
        // Operation failed; error is an HttpErrorResponse
        error => status = 'failed'
      ),
      finalize(() => {
        const elapsed = Date.now() - started;
        if (typeof window === 'undefined') {
          console.log('Request performance: ', request.url, `${elapsed} ms`);
          this.state.set(key, <HttpReturnRecord>{
            url: request.url,
            status,
            ssr: true,
            responseTimeInMillis: elapsed,
            completeTime: Date.now(),
            formattedCompleteTime: null
          });
        } else {
          this.store.dispatch(new EventTransceiverReceiveHTTPReturnEvent({
            url: request.url,
            status,
            ssr: false,
            responseTimeInMillis: elapsed,
            completeTime: Date.now(),
            formattedCompleteTime: null
          }));
        }
      })
    );
  }
}
