import {ProductItem} from '../modules/cart/models/cart.interface';
import {Address, DeliveryPointOfService, PickupEntry, PickupOrderGroupItem} from '../modules/cart/models/pick-up.interface';

/**
 * Group methods that will need to use in Schedule Delivery and Ship To You product list
 * getDeliveryOrderGroupDataIndex: return index number of the item in the product list
 * getDeliverOrderGroupData: return delivery order group data
 * */
export const deliveryOrderGroup = {
  /**
   * Get delivery order group index data which referred by `deliveryOrderType`: BODFS / STH
   * param {object} productList - cart product object contained product list
   * param {string} deliveryOrderType - string product order type for example BODFS or STH
   * return {number} index number
   * */
  getDeliveryOrderGroupDataIndex: function (productList: ProductItem, deliveryOrderType: string): number {
    if (!productList.deliveryOrderGroups) {
      return -1;
    }
    const index: number = productList.deliveryOrderGroups.findIndex(item => item.deliveryOrderType === deliveryOrderType);
    return index;
  },

  /**
   * Get deliver order group data which can be STH or BODFS based on `deliveryOrderType`
   * param {object} productList - cart product object contained product list
   * param {string} deliveryOrderType - string product order type for example STH or BODFS
   * return {object} BODFS or STH object product data
   * */
  getDeliverOrderGroupData: function (productList: ProductItem, deliveryOrderType: string): PickupEntry[] {
    const index: number = this.getDeliveryOrderGroupDataIndex(productList, deliveryOrderType);
    if (index > -1) {
      const productEntry: PickupEntry[] = productList.deliveryOrderGroups[index].entries;
      return productEntry;
    }
  },
  /**
   * Get deliver order group object which can be STH or BODFS based on `deliveryOrderType`
   * param {object} productList - cart product object contained product list
   * param {string} deliveryOrderType - string product order type for example STH or BODFS
   * return {object} BODFS or STH object product data
   * */
  getDeliverOrderObject: function (productList: ProductItem, deliveryOrderType: string): any {
    const index: number = this.getDeliveryOrderGroupDataIndex(productList, deliveryOrderType);
    if (index > -1) {
      const deliveryOrder = productList.deliveryOrderGroups[index];
      return deliveryOrder;
    }
  }
};

/**
 * Group methods that will need to use in Pick Up Today and Pick Up Later product list
 * getPickUpOrderGroupDataIndex: return index number of the item in the product list
 * getPickUpOrderGroupData: return pick up order group data
 * getDeliveryPointOfService: return store pick up location information
 * */
export const pickUpOrderGroup = {
  /**
   * Get pick up order groupd index data which referred by `pickupOrderType`: BOPIS / BOSS
   * param {object} productList - cart product object contained product list
   * param {string} pickupOrderType - string product order type for example BOPIS or BOSS
   * return {number} index number
   * */
  getPickUpOrderGroupDataIndex: function (productList: ProductItem, pickupOrderType: string): number {
    if (!productList.pickupOrderGroups) {
      return -1;
    }
    const index: number = productList.pickupOrderGroups.findIndex(item => item.pickupOrderType === pickupOrderType);
    return index;
  },
  /**
   * Get pick up order groupd data which can be POPIS or BOSS based on `pickupOrderType`
   * param {object} productList - cart product object contained product list
   * param {string} pickupOrderType - string product order type for example BOPIS or BOSS
   * return {object} BOPIS or BOSS object product data
   * */
  getPickUpOrderGroupData: function (productList: ProductItem, pickupOrderType: string): PickupEntry[] {
    const index: number = this.getPickUpOrderGroupDataIndex(productList, pickupOrderType);
    if (index > -1) {
      const productEntry: PickupEntry[] = productList.pickupOrderGroups[index].entries;
      return productEntry;
    }
  },

  getPickUpOrderGroupObject: function (productList: ProductItem, pickupOrderType: string): PickupOrderGroupItem {
    const index: number = this.getPickUpOrderGroupDataIndex(productList, pickupOrderType);
    if (index > -1) {
      const productEntry = productList.pickupOrderGroups[index];
      return productEntry;
    }
  },
  /**
   * Get delivery point of service which is pick up store
   * param {object} productList - cart product object contained product list
   * param {string} pickupOrderType - string product order type for example BOPIS or BOSS
   * return {object} - object that contains {
   *                      - address: pick up store address
   *                      - name: store number for example #7073
   *                      - displayName: store name for example LEASIDE
   * }
   * */
  getDeliveryPointOfService: function (productList: ProductItem, pickupOrderType: string): DeliveryPointOfService {
    const productEntry: PickupEntry[] = this.getPickUpOrderGroupData(productList, pickupOrderType);
    if (productEntry && productEntry.length) {
      // backend confirmed that the `entries` array object is always contained 1 object
      const deliveryStore = productEntry[0].deliveryPointOfService;
      const address: Address = {
        formattedAddress: deliveryStore.address.formattedAddress,
        phone: deliveryStore.address.phone
      };
      const deliveryPOS: DeliveryPointOfService = {
        address: address,
        name: deliveryStore.name,
        displayName: deliveryStore.displayName,
        openingHours: deliveryStore.openingHours,
        geoPoint: deliveryStore.geoPoint
      };

      return deliveryPOS;
    }
  }
};
