import { Component, Input, Output, EventEmitter, OnChanges } from '@angular/core';
import { DateTimeService} from '../../services/date-time/date-time.service';
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'date-picker',
  templateUrl: './date-picker.component.html'
})
export class DatePickerComponent implements OnChanges {
    @Input() selectedDate: Date;
    @Input() availableDates: Date[];
    @Input() displayMonth: Date;
    @Output() doneClicked: EventEmitter<Date> = new EventEmitter();

    prevButtonDisabled = false;
    nextButtonDisabled = false;
    weeks: [Date[]];
    monthHeader: string;

    constructor(private dateTimeSvc: DateTimeService, private translateSvc: TranslateService) {}

    ngOnChanges() {
        if (this.dateTimeSvc.isDate(this.displayMonth)) {
            this.renderCalendar();
        }
    }

    renderCalendar() {
        this.createCalendarMonth(this.displayMonth);
        this.disablePrevButton(this.displayMonth);
        this.disableNextButton(this.displayMonth);
        const monthKey = this.dateTimeSvc.monthKeys[this.dateTimeSvc.getMonth(this.displayMonth)];
        this.translateSvc.get(`MONTHS.${monthKey}`).subscribe((translatedMonth: string) => {
            this.monthHeader = translatedMonth;
        });
    }

    isDateToday(date) {
        return this.dateTimeSvc.isToday(date);
    }

    isDateSelected(date) {
        return this.dateTimeSvc.isEqual(this.selectedDate, date);
    }

    disablePrevButton(month) {
        const earliestMonth = this.dateTimeSvc.startOfMonth(this.availableDates[0]);
        if (this.dateTimeSvc.differenceInMonths(this.dateTimeSvc.startOfMonth(this.dateTimeSvc.getPrevMonth(month)), earliestMonth) < 0) {
            this.prevButtonDisabled = true;
        } else {
            this.prevButtonDisabled = false;
        }
    }

    disableNextButton(month) {
        const lastPossibleMonth = this.dateTimeSvc.startOfMonth(this.availableDates[this.availableDates.length - 1]);
        if (this.dateTimeSvc.differenceInMonths(this.dateTimeSvc.startOfMonth(this.dateTimeSvc.getNextMonth(month)), lastPossibleMonth) > 0) {
            this.nextButtonDisabled = true;
        } else {
            this.nextButtonDisabled = false;
        }
    }

    showPrevMonth() {
        this.displayMonth = this.dateTimeSvc.getPrevMonth(this.displayMonth);
        this.renderCalendar();
    }

    showNextMonth() {
        this.displayMonth = this.dateTimeSvc.getNextMonth(this.displayMonth);
        this.renderCalendar();
    }

    onDateClick(day) {
        if (!this.isDateDisabled(day)) {
            this.selectedDate = day;
            const selectedMonth = this.dateTimeSvc.getMonth(this.selectedDate);
            const selectYear = this.dateTimeSvc.getYear(this.selectedDate);
            const displayMonth = this.dateTimeSvc.getMonth(this.displayMonth);
            const displayYear = this.dateTimeSvc.getYear(this.displayMonth);
            if (selectYear < displayYear || (selectYear === displayYear && selectedMonth < displayMonth)) {
                this.showPrevMonth();
            }
            if (selectYear > displayYear || (selectYear === displayYear && selectedMonth > displayMonth)) {
                this.showNextMonth();
            }
        }
    }

    handleDoneClicked() {
        this.doneClicked.emit(this.selectedDate);
    }

    createCalendarMonth(month) {
        this.weeks = [[]];
        const days = this.getDatesToRender(month);
        let week = [];
        days.forEach((day) => {
            week.push(day);
            if (week.length === 7) {
                this.weeks.push(week);
                week = [];
            }
        });
    }

    getDatesToRender(month) {
        const firstDateToRender = this.dateTimeSvc.startOfWeek(this.dateTimeSvc.startOfMonth(month));
        const lastDateToRender = this.dateTimeSvc.endOfWeek(this.dateTimeSvc.endOfMonth(month));
        return this.dateTimeSvc.eachDay(firstDateToRender, lastDateToRender);
    }

    isDateDisabled(date) {
        return this.availableDates.find((availableDate) => {
            return this.dateTimeSvc.isEqual(availableDate, date);
        }) === undefined;
    }
}
