import { Injectable } from '@angular/core';
import * as models from '../models/adobe-analytics-data-layer-models.interface';
import * as constants from '../constants/adobe-analytics.constants';
import { UserInfoService, LanguageService } from '@hdca/fe-angular-shared';
import { OptimizedPrice } from '../models/product-localized.interface';

/**
 * Generates payloads for Adobe DTM
 */
@Injectable()
export class AdobeAnalyticsPayloadService {

  constructor(
    private user: UserInfoService,
    private langService: LanguageService
  ) { }

  private getLanguage() {
    return this.langService.getLanguage() === 'fr'
      ? constants.languageTypes.fr
      : constants.languageTypes.en;
  }

  // formats string value of price as a proper number
  private formatPrice(analyticsPrice: string) {
    return this.langService.getLanguage() === 'fr'
    ? analyticsPrice.replace(/\s/g, '').replace(/,/g, '.').replace(/[$]+/g, '')
    : analyticsPrice.replace(/[$,]+/g, '');
  }

  private getTimestamp() {
    return Date.now();
  }

  private returnPageDetail(event: models.EventBlock, productUnlocalizedData, SearchRedirectBlock, pageMeta) {
    // Default condition is PIP (product information page)
    // Each page will have a dedicated page detail function
    switch (event.page) {
      case constants.pageTypes.nullSearch:
        return this.returnSearchPageDetail(SearchRedirectBlock, pageMeta);
      case constants.pageTypes.contactUs:
        return pageMeta;
      default: {
        if (!!productUnlocalizedData) {
          return this.returnProductPageDetail(productUnlocalizedData, SearchRedirectBlock);
        } else {
          // In event of no unlocalized data, return basic data set (no name or breadcrumbs, everything not obtained from unlocalized data)
          return {
            'pageName': '',
            'pageSiteSection': constants.pageTypes.pip,
            'sysEnv': constants.systemEnvironments.desktop,
            'pageContentType': constants.pageTypes.pip,
            'search': SearchRedirectBlock
          };
        }
      }
    }
  }

  private returnProductPageDetail(productUnlocalizedData, SearchRedirectBlock) {
    // Processing dedicated to Product page specific data
    // "breadcrumbs" are provided by the Product data payload itself: "productUnlocalizedData"
    const breadcrumbArray = (productUnlocalizedData.breadCrumbs && productUnlocalizedData.breadCrumbs.length >= 0) ? productUnlocalizedData.breadCrumbs : [];  // runtime check
    return {
      'pageName': `${productUnlocalizedData.manufacturer} ${productUnlocalizedData.name}`,
      'pageSiteSection': constants.pageTypes.pip,
      'sysEnv': constants.systemEnvironments.desktop,
      'pageContentType': constants.pageTypes.pip,
      'breadcrumbs': productUnlocalizedData.breadCrumbs,
      'pageCategory': breadcrumbArray[0] ? breadcrumbArray[0].title : '',
      'pageCategory1': breadcrumbArray[1] ? breadcrumbArray[1].title : '',
      'pageCategory2': breadcrumbArray[2] ? breadcrumbArray[2].title : '',
      'pageCategory3': breadcrumbArray[3] ? breadcrumbArray[3].title : '',
      'pageCategory4': breadcrumbArray[4] ? breadcrumbArray[4].title : '',
      'pageCategory5': breadcrumbArray[5] ? breadcrumbArray[5].title : '',
      'search': SearchRedirectBlock
    };
  }

  private returnSearchPageDetail(SearchRedirectBlock, pageMeta) {
    // Processing dedicated to Search Page (current Null Search Page) specific data
    // - Future: Null Search Page will simply become search page, all "null search" specific text will need replacement at that time
    // - "breadcrumbs" are provided by the page directly via "pageMeta" object
    // - "pageName" is supplied via the "pageMeta" object
    const breadcrumbArray = pageMeta.breadcrumbs ? pageMeta.breadcrumbs : [];  // runtime check
    return {
      'pageName': `${pageMeta.pageName}`,
      'pageSiteSection': constants.pageTypes.nullSearch,
      'sysEnv': constants.systemEnvironments.desktop,
      'pageContentType': constants.pageTypes.nullSearch,
      'breadcrumbs': pageMeta.breadcrumbs,
      'pageCategory': breadcrumbArray[0] ? breadcrumbArray[0].title : '',
      'pageCategory1': breadcrumbArray[1] ? breadcrumbArray[1].title : '',
      'pageCategory2': breadcrumbArray[2] ? breadcrumbArray[2].title : '',
      'pageCategory3': breadcrumbArray[3] ? breadcrumbArray[3].title : '',
      'pageCategory4': breadcrumbArray[4] ? breadcrumbArray[4].title : '',
      'pageCategory5': breadcrumbArray[5] ? breadcrumbArray[5].title : '',
      'search': SearchRedirectBlock
    };
  }

  private returnProduct(productUnlocalized, productLocalized, fbtData) {
    let eligibility = '', productInfo;

    if (!!productUnlocalized && !!productLocalized) {
      if (productUnlocalized.bopis) {
        eligibility = eligibility + ':BOPIS';
      }
      if (productLocalized.fulfillmentMessages && productLocalized.fulfillmentMessages.availableMessages && productLocalized.fulfillmentMessages.availableMessages.boss) {
        eligibility = eligibility + ':BOSS';
      }
      if (productUnlocalized.stock && productUnlocalized.stock.stockLevelStatus && productUnlocalized.stock.stockLevelStatus === 'outOfStock') {
        eligibility = eligibility + ':OOS';
      }
      if (productUnlocalized.sellableIntent === 'S' || productUnlocalized.sellableIntent === 'B') {
        eligibility = eligibility + ':INSO';
      }
      if (productLocalized.fulfillmentMessages && productLocalized.fulfillmentMessages.availableMessages && productLocalized.fulfillmentMessages.availableMessages.shipToHome) {
        eligibility = eligibility + ':STH';
      }
      if (eligibility.length > 0) {
        eligibility = eligibility.substr(1);
      }

      productInfo = {
        'productId': productUnlocalized.code,
        'attributes': {
          'eligible': eligibility,
          'freeShipping': productUnlocalized.freeShipping,
          'rating': productUnlocalized.averageRating,
          'reviews': productUnlocalized.numberOfReviews,
          'questions': productUnlocalized.numberOfQuestions,
          'storeShippingAvailability': (!!productUnlocalized.boss && !!productUnlocalized.shipToStoreTime) ? constants.stockAvailabilityTypes.inStock : constants.stockAvailabilityTypes.outOfStock,
        },
        'superSku': {
          'eligible': !!(productUnlocalized.variantOptionsSorted && productUnlocalized.variantOptionsSorted.length),
          'attributeCount': productUnlocalized.variantOptionsSorted ? productUnlocalized.variantOptionsSorted.length : 0,
          'attributes': productUnlocalized.variantOptionsSorted ? this.getSuperSku(productUnlocalized.variantOptionsSorted) : [],
        },
        'relatedSearches': !!(productUnlocalized.relatedLinks && productUnlocalized.relatedLinks.relatedSearches) ? productUnlocalized.relatedLinks.relatedSearches : [],
        'relatedProducts': !!(productUnlocalized.relatedLinks && productUnlocalized.relatedLinks.relatedProducts) ? productUnlocalized.relatedLinks.relatedProducts : []
      };
    }

    if (!!productLocalized) {
      productInfo = {
        ...productInfo,
        'attributes': {
          ...productInfo.attributes,
          'localizedAssortment': (!!productLocalized.fulfillmentMessages.availabilityMessage) ? productLocalized.fulfillmentMessages.availabilityMessage : '',  // unavailable in unlocalized
         'regPrice': (productLocalized.optimizedPrice && productLocalized.optimizedPrice.regprice) ? this.formatPrice(productLocalized.optimizedPrice.regprice.formattedValue) : '',  // unavailable in unlocalized
         'activePrice': productLocalized.optimizedPrice ? this.formatPrice(this.getActivePrice(productLocalized.optimizedPrice)) : '', // unavailable in unlocalized
          'storeStockAvailability': productLocalized.storeStock ? productLocalized.storeStock.stockLevelStatus : '',  // unavailable in unlocalized
          'onlineStockAvailability': productLocalized.onlineStock ? productLocalized.onlineStock.stockLevelStatus : '', // unavailable in unlocalized
        },
      };
    }

    if (fbtData && fbtData.fbt && (fbtData.fbt.supportingArticles || fbtData.fbt.anchorArticle)) {
        productInfo = {
          ...productInfo,
          'fbt': {
            'item': this.getFbt(fbtData.fbt),
          }
        };
    }

    return productInfo;
  }

  private getActivePrice(optimizedPrice: OptimizedPrice): string {
    return optimizedPrice.promoprice ?
      optimizedPrice.promoprice.formattedValue :
      optimizedPrice.regprice ?
      optimizedPrice.regprice.formattedValue : '';
  }

  private getSuperSku(variant) {
    return variant.map(item => {
      const selectedSku = item.selectedVariantOptionQualifier;
      const otherSku = item.variantOptionQualifiers;
      return {
        'attributeId': (selectedSku && selectedSku.variantValueCategoryCode) ? selectedSku.variantCategoryCode : '',
        'attributeName': item.name,
        'valueIds': otherSku ? this.getSkuId(otherSku, 'variantValueCategoryCode') : '',
        'valueNames': otherSku ? this.getSkuId(otherSku, 'variantCategoryCode') : '',
        'activeValueId': (selectedSku && selectedSku.variantCategoryCode) ? selectedSku.variantValueCategoryCode : '',
        'activeValueName': selectedSku ? selectedSku.value : '',
      };
    });
  }

  private getSkuId(sku, key) {
    const valueIds = [];
    sku.map(item => {
      if (item[key]) {
        valueIds.push(item[key]);
      }
    });
    return valueIds;
  }

  private getFbt(fbt) {
    const fbtSupport = (fbt.supportingArticles && fbt.supportingArticles.length) ? fbt.supportingArticles : '';
    const fbtAnchor = (fbtSupport.length > 0) ? [fbt.anchorArticle].concat(fbtSupport) : [fbt.anchorArticle];

    if (!!fbtAnchor) {
      return fbtAnchor.map((item, index) => {
        return {
          'slotLocation': index + 1,
          'productId': item.code,
          'sellPrice': item.optimizedPrice.displayPrice.value,
        };
      });
    }
  }

  private getPetaInfo(petaItem, value) {
    const peta = petaItem.filter(item => {
      return (item.FULFILLMENT_TYPE === value);
    })[0];
    return peta ? `${peta.FULFILLMENT_ETA} days` : '';
  }

  private returnCart(cart) {
    const cartItem = this.getCartItem(cart);
    return {
      'active': !!cart,
      'id': cart.guid,
      'item': cartItem.item,
      'attributes': {
        'totalUnits': cart.totalItemsQuantity,
        'shipping': {
          'mixed': cartItem.mixed,
        },
        'productcount': cart.numberOfProducts,
        'deliveryCost': !!cart.totalDeliveryCost ? cart.totalDeliveryCost.value : '',
      },
    };
  }

  private getCartItem(cart) {
    const cartItem = {
      item: [],
      mixed: []
    };
    const appln = cart.applianceOrderGroups &&  cart.applianceOrderGroups.length ? cart.applianceOrderGroups : [];
    const delivery = cart.deliveryOrderGroups &&  cart.deliveryOrderGroups.length ? cart.deliveryOrderGroups : [];
    const pickup = cart.pickupOrderGroups &&  cart.pickupOrderGroups.length ? cart.pickupOrderGroups : [];
    let productType = {};

    if (delivery.length) {
      const deliveryShipping = this.getCartShippingType(delivery, 'deliveryOrderType');
      productType = {...productType, ...deliveryShipping};
    }

    if (pickup.length) {
      const pickupShipping = this.getCartShippingType(pickup, 'pickupOrderType');
      productType = {...productType, ...pickupShipping};
    }

    if (appln.length) {
      const applnShipping = this.getCartShippingType(appln);
      productType = {...productType, ...applnShipping};
    }

    Object.keys(productType).map(data => {
      if (cartItem.mixed.indexOf(productType[data]) === -1) {
        cartItem.mixed.push(productType[data]);
      }
    });

    if (cart.entries && cart.entries.length) {
      cartItem.item = cart.entries.map(item => {
        return {
          'productId': item.product.code,
          'price': {
            'baseprice': (item.product.optimizedPrice && item.product.optimizedPrice.displayPrice) ? item.product.optimizedPrice.displayPrice.value : '',
            'discountPrice': (item.product.optimizedPrice && item.product.optimizedPrice.regprice) ? item.product.optimizedPrice.regprice.value : '',
            'voucherCode': item.promotionalSavings ? item.promotionalSavings.value : '',
            'discount': 1,
            'shippingMethod': productType[item.product.code],
          },
          'quantity': item.quantity,
          'itemTotal': item.basePriceTotal ? item.basePriceTotal.value : '',
          'deliveryDays': item.deliveryTime,
        };
      });
    }
    return cartItem;
  }

  private getCartShippingType(data, type?) {
    const shipping = {};
    data.map(product => {
      product.entries.map(item => {
        shipping[item.product.code] = type ? product[type] : 'APPL';
      });
    });
    return shipping;
  }

  private returnUser(user) {
    return {
      'guestStatus': !!user.email ? constants.userGuestStatusTypes.registered : constants.userGuestStatusTypes.anonymous,
      'profile': {
        'profileInfo': {
          'email': user.email,
          'language': this.getLanguage(),
          'profileId': user.authorization,
          'type': !!user.proxtraUserLoggedin ? constants.userProfileTypes.proxtra : (!!user.associateUserLoggedin ? constants.userProfileTypes.associate : constants.userProfileTypes.regular),
        }
      }
    };
  }

  public formClickDataPayload(event: models.EventBlock, elementType: constants.clickElementTypes, elementSubType: constants.clickElementSubTypes, elementValue: constants.clickElementValues ): models.ElementClickPayload {
     const payload: models.ElementClickPayload = {
        'event': {
            'type': event.type,
            'page': event.page,
            'timestamp': this.getTimestamp(),
            'language': this.getLanguage()
        },
        elemenntClickBlock: {
          elementType,
          elementSubType,
          elementValue
        }
      };
      const userAccountData = this.user.getUserAccountData();
      if (!!userAccountData) {
        payload.user = this.returnUser(userAccountData);
      }
      return payload;
  }

  public returnCertona(certona, pageType) {
    // Return certona data depending on the page type
    // PENDefault is product (PIP) - accessories, collections, trending - PENDING, return {} for now
    switch (pageType) {
      case constants.pageTypes.nullSearch:
        return {
          'recommendedProductList': certona
        };
      default: {
        // Condition for Product Page (PIP)
        return {
          'accessoriesProductList': certona.accessories,
          'collectionProductList': certona.collection,
          'trendingProductList': certona.trending
        };
      }
    }
  }

  public formPageLoadPayload(event: models.EventBlock, dataWrapper: models.DataWrapperBlock, pageMeta): models.PageAndModalViewPayload {
    // Decompose data wrapper
    const productUnlocalized = dataWrapper.productUnlocalized;
    const fbtData = dataWrapper.fbtData;
    const redirect = dataWrapper.redirect;
    const productLocalized = dataWrapper.productLocalized;
    const cartData = dataWrapper.cartData;
    const certona = dataWrapper.certona;
    const nullSearchPage = dataWrapper.nullSearchData;
    // Begin payload
    const payload: models.PageAndModalViewPayload = {
      'event': {
        'type': event.type,
        'timestamp': this.getTimestamp(),
        'page': event.page,
        'language': this.getLanguage()
      },
      'pageDetail': this.returnPageDetail(event, productUnlocalized, redirect, pageMeta)
    };
    if (!!productUnlocalized && !!productLocalized) {
      payload.productOverview = this.returnProduct(productUnlocalized, productLocalized, fbtData);
    }
    const userAccountData = this.user.getUserAccountData();
    if (!!userAccountData) {
      payload.user = this.returnUser(userAccountData);
    }
    if (!!cartData && cartData.guid && !cartData.error.errorObj) {
      payload.cart = this.returnCart(cartData);
    }
    if (!!certona) {
      payload.certona = this.returnCertona(certona, event.page);
    }
    if (!!nullSearchPage) {
      payload.nullSearchPage = nullSearchPage.relatedQueries;
    }
    if (event.page === constants.pageTypes.contactUs) {
      payload.user = this.returnUser(userAccountData);
    }
    return payload;
  }

  public formSiteErrorPayload(event: models.EventBlock, errorType: constants.siteErrorTypes, userInformed: boolean, errorCode: string, errorValue: string, target: string, errorPayload: any): models.SiteErrorPayload {
    const payload: models.SiteErrorPayload = {
      'event': {
        'type': event.type,
        'timestamp': this.getTimestamp(),
        'page': event.page,
        'language': this.getLanguage()
      },
      error: {
        errorType,
        userInformed,
        errorCode,
        errorValue,
        target,
        errorPayload,
      }
    };

    const userAccountData = this.user.getUserAccountData();
    if (!!userAccountData) {
      payload.user = this.returnUser(userAccountData);
    }
    return payload;
  }

  public formAddToCartPayload(event: models.EventBlock, items: models.AddToCartInfoBlock[]): models.AddToCartPayload {
    const payload: models.AddToCartPayload = {
      'event': {
        'type': event.type,
        'timestamp': this.getTimestamp(),
        'page': event.page,
        'language': this.getLanguage()
      },
      items
    };
    const userAccountData = this.user.getUserAccountData();
    if (!!userAccountData) {
      payload.user = this.returnUser(userAccountData);
    }
    return payload;
  }

  public formSuperSkuSwitchPayload(
    page: constants.pageTypes,
    productId: string,
    attributeName: string,
    attributeId: string,
    targetProductId: string,
    targetValueId: string,
    targetValueName: string): models.SuperSkuSwitchPayload {
      const payload: models.SuperSkuSwitchPayload = {
        'event': {
          'type': constants.eventTypes.superSkuSwitch,
          'page': page,
          'timestamp': this.getTimestamp(),
          'language': this.getLanguage()
        },
        superSkuSwitchData : {
          productId,
          attributeName,
          attributeId,
          targetProductId,
          targetValueId,
          targetValueName
        }
      };
      const userAccountData = this.user.getUserAccountData();
      if (!!userAccountData) {
        payload.user = this.returnUser(userAccountData);
      }
      return payload;
  }

  public formCarouselItemClickPayload(productId: string,
                                      contextProductId: string,
                                      slotLocation: number,
                                      productInteractionLocation: constants.productInteractionLocationTypes,
                                      productInteractionPageType: constants.pageTypes,
                                      productInteractionLocationTitle: string,
                                      productInteractionLocationOrder: number): models.CarouselProductClickPayload {

    const payload: models.CarouselProductClickPayload = {
      productId,
      contextProductId,
      slotLocation,
      productInteractionLocation,
      productInteractionPageType,
      productInteractionLocationTitle,
      productInteractionLocationOrder,
      'event': {
        'type': constants.eventTypes.carouselItemClick,
        'page': productInteractionPageType,
        'timestamp': this.getTimestamp(),
        'language': this.getLanguage()
      }
    };
    const userAccountData = this.user.getUserAccountData();
    if (!!userAccountData) {
      payload.user = this.returnUser(userAccountData);
    }
    return payload;
  }

  public formPETAUpdate(petaItem: any) {
    const payload: models.PETAPayload = {
      'event': {
        'type': constants.eventTypes.pipPETA,
        'timestamp': this.getTimestamp(),
        'page': constants.pageTypes.pip,
        'language': this.getLanguage()
      },
    };
    if (!!petaItem.length) {
      payload.peta = {
        'deliveryDays': {
          'sth': this.getPetaInfo(petaItem, 'SHIPTOHOME'),
          'boss': this.getPetaInfo(petaItem, 'BOSS'),
        }
      };
    }
    const userAccountData = this.user.getUserAccountData();
    if (!!userAccountData) {
      payload.user = this.returnUser(userAccountData);
    }
    return payload;
  }
}
